#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import createObject, ComponentLookupError, getUtility
from zope.dublincore.interfaces import IZopeDublinCore
from zope.app.container.constraints import checkObject
from zope.app.container.interfaces import INameChooser
from zope.app.interface import queryType

from zc.table import column

from z3c.form import button, field
from z3c.formui import form

from zw.contentsform.adding import AddingFormMixin
from zw.contentsform.column import FieldColumn, ContentNameColumn, Columns, \
    FormActionsColumn, ProviderColumnBase
from zw.contentsform.contentsform import OrderableContentsForm
from zw.contentsform.cutcopypaste import CutCopyPasteMixin, RenamableMixin

from zw.vogon.attribute import interfaces


class ManageAttributeCategories(AddingFormMixin, CutCopyPasteMixin, 
                                RenamableMixin, OrderableContentsForm):
    form.extends(AddingFormMixin, CutCopyPasteMixin, RenamableMixin)

    canCut = False
    canCopy = False
    canPaste = False
    canDelete = True

    label = _( u'label-attribute.browser.ManageAttributeCategories.label',
               u"Manage attribute categories" )

    cssClasses = {'table': 'listing' }

    @Lazy
    def container(self):
        return interfaces.IAttributeTemplates(self.context)
    
    def _getColumns(self):
        return Columns(
            ContentNameColumn( title=u'Unique name' ),
            FieldColumn( interfaces.IAttributeCategory['title'], action='contents.html' ),
            FieldColumn( interfaces.IAttributeCategory['description'] ),
            FieldColumn( interfaces.IAttributeCategory['public'] ),
            #FormActionsColumn() # XXX 
            )
            
    @button.buttonAndHandler( _( u'Add', ), name='add_new',
                              condition = lambda form: \
                                  form.session.get('add') is None )
    def handleAddNew(self, action):
        self.session['add'] = u'zw.vogon.attribute.Category'
        self.request.response.redirect(self.request.getURL())




class TemplateTypeColumn(column.Column):

    title = _( u"Attribute type" )
    name = u'template_type'

    def __init__(self):
        pass

    def renderCell(self, item, formatter):
        iface = queryType(item.context, interfaces.IAttributeType)
        return iface.__name__

class ManageTemplateAttributes(CutCopyPasteMixin, 
                               RenamableMixin, OrderableContentsForm):
    form.extends(CutCopyPasteMixin, RenamableMixin)

    canCut = False
    canCopy = False
    canPaste = False
    canDelete = True

    @property
    def label(self):
        return _( u'label-attribute.browser.ManageTemplateAttributes.label',
                  u"Attribute category '${name}'",
                  mapping = {'name': self.context.title } )

    cssClasses = {'table': 'listing' }
    
    def update(self):
        addAction = self.request.get('type_name', None)
        if addAction is not None:
            self.request.response.redirect(
                "%s/add.html?type_name=%s" % ( self.request.getURL(1), addAction ) )
        super(ManageTemplateAttributes, self).update()
            

    def _getColumns(self):
        return Columns(
            #ContentNameColumn( title=u'Unique name' ),
            FieldColumn( interfaces.IAttributeTemplate['title'], action='edit.html' ),
            FieldColumn( interfaces.IAttributeTemplate['description'] ),
            TemplateTypeColumn(),
            #FormActionsColumn() # XXX 
            )
            
class TemplateAddForm(form.AddForm):
    
    label = _( u"Add attribute field" )
    
    def __init__(self, context, request):
        super(TemplateAddForm, self).__init__(context, request)
        factoryname = request.get('type_name', None)
        try:
            obj = createObject(factoryname)
        except ComponentLookupError:
            self.request.response.redirect(self.request.getURL(1))
            return

        self.factoryname = factoryname
        schema = queryType(obj, interfaces.IAttributeType)
        self.fields = field.Fields(schema).omit('__parent__', '__name__')

    def create(self, data):
        obj = createObject(self.factoryname)
        changes = form.applyChanges(self, obj, data)
        return obj

    def add(self, content, name=None):
        name = getattr(content, '__name__', name)
        chooser = INameChooser(self.context)

        # check precondition
        checkObject(self.context, name, content)

        name = chooser.chooseName(name or '', content)
        chooser.checkName(name, content)

        self.context[name] = content
        return self.context[name]

    def nextURL(self):
        return self.request.getURL(1)


class TemplateEditForm(form.EditForm):

    label = _( u"Edit attribute field" )

    def __init__(self, context, request):
        super(TemplateEditForm, self).__init__(context, request)
        schema = queryType(context, interfaces.IAttributeType)
        self.fields = field.Fields(schema).omit('__parent__', '__name__')

