#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from persistent import Persistent

from zope import schema
from zope.component import adapter, adapts
from zope.interface import implementer, implements
from zope.schema.fieldproperty import FieldProperty
from zope.app.container.contained import Contained
from zope.app.interface import queryType

from zw.vogon.attribute import interfaces

class AttributeField(object):
    implements(interfaces.IAttributeField)
    
    def __init__(self, template):
        iface = queryType(template, interfaces.IAttributeType)
        args = ()
        kw = {}
        for name, field in schema.getFields(iface).items():
            kw[name] = getattr(template, name, field.default)

        # Throw away the __parent__ field
        kw.pop('__parent__', None)
        
        self._process_kw(kw)

        self.public = kw.pop('public')
        self.readonly = kw.pop('readonly')
        super(AttributeField, self).__init__(*args, **kw)

    def _process_kwargs(self, kw):
        pass



class Bool(AttributeField, schema.Bool):
    implements(interfaces.IBool)
    adapts(interfaces.IBoolTemplate)


class Text(AttributeField, schema.Text):
    implements(interfaces.IText)

    def _process_kwargs(self, kw):
        self.cols = kw.pop('cols')
        self.rows = kw.pop('rows')

class TextLine(AttributeField, schema.TextLine):
    implements(interfaces.ITextLine)

    def _process_kwargs(self, kw):
        self.cols = kw.pop('cols')
        kw.pop('rows', None)

@adapter(interfaces.ITextTemplate)
@implementer(interfaces.IAttributeField)
def textField(template):
    if template.rows == 1:
        return TextLine(template)
    return Text(template)
        

class Int(AttributeField, schema.Int):
    implements(interfaces.IInt)
    adapts(interfaces.IIntTemplate)


class Datetime(AttributeField, schema.Datetime):
    implements(interfaces.IDatetime)

class Date(AttributeField, schema.Date):
    implements(interfaces.IDate)

class Time(AttributeField, schema.Time):
    implements(interfaces.ITime)

class Timedelta(AttributeField, schema.Timedelta):
    implements(interfaces.ITimedelta)

@adapter(interfaces.IDatetimeTemplate)
@implementer(interfaces.IAttributeField)
def datetimeField(template):
    return Date(template)


class URI(AttributeField, schema.URI):
    implements(interfaces.IURI)
    adapts(interfaces.IURITemplate)

    def _process_kwargs(self, kw):
        self.cols = kw.pop('cols')
        kw.pop('rows', None)


class RFC822MailAddress(AttributeField, schema.URI):
    implements(interfaces.IRFC822MailAddress)
    adapts(interfaces.IRFC822MailAddressTemplate)

    def _process_kwargs(self, kw):
        self.cols = kw.pop('cols')
        kw.pop('rows', None)


class Object(AttributeField, schema.Object):
    implements(interfaces.IObject)
    adapts(interfaces.IObjectTemplate)
