#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from persistent import Persistent

from zope.annotation.factory import factory
from zope.component import adapter, adapts
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.api import getParent

from zope.app.container.contained import Contained
from zope.app.container.ordered import OrderedContainer
from zope.app.container.interfaces import IObjectRemovedEvent
from zope.app.interface import PersistentInterfaceClass

from z3c.traverser.traverser import AdapterTraverserPlugin

from zw.vogon.attribute import interfaces


class AttributeCategory(OrderedContainer):
    implements(interfaces.IAttributeCategory, 
               interfaces.IAttributeTemplateContainer)

    title = FieldProperty(interfaces.IAttributeCategory['title'])
    description = FieldProperty(interfaces.IAttributeCategory['description'])
    public = FieldProperty(interfaces.IAttributeCategory['public'])

    def getAttributeSchema(self):
        attrs = {}
        for name, item in self.items():
            attrs[str(name)] = AttributeField(
                title = item.title,
                description = item.description,
                required = item.required,
                public = item.public,
                default = item.default )
        return PersistentInterfaceClass(
            name = 'IAttributeSchema',
            bases = (interfaces.IBaseAttributeSchema, ),
            attrs = attrs,
            __doc__ = """The customized attributes schema.""",
            __module__ = 'zw.vogon.attribute' )


class AttributeTemplates(OrderedContainer):
    implements(interfaces.IAttributeTemplates,
               interfaces.IAttributeCategoryContainer)
    adapts(interfaces.IStoreAttributeTemplates)

    title = _( u"Attribute templates" )

    @property
    def attributes(self):
        return interfaces.IAttributes(getParent(self), None)


attributeTemplatesFactory = factory(AttributeTemplates,
                                    'attribute-templates')

attributeTemplateTraverserPlugin = AdapterTraverserPlugin(
    u'attribute-templates',  interfaces.IAttributeTemplates)
                         

@adapter(interfaces.IStoreAttributeTemplates, IObjectRemovedEvent)
def removeAttributeTemplates(context, event):
    folder = interfaces.IAttributeTemplates(context)
    for name, item in folder.items():
        uncontained(item, folder, name)

##############################################################################


class AttributeTemplate(Contained, Persistent):
    implements(interfaces.IAttributeTemplate)

    title = FieldProperty(interfaces.IAttributeTemplate['title'])
    description = FieldProperty(interfaces.IAttributeTemplate['description'])
    required = FieldProperty(interfaces.IAttributeTemplate['required'])
    public = FieldProperty(interfaces.IAttributeTemplate['public'])
    readonly = FieldProperty(interfaces.IAttributeTemplate['readonly'])
    default = FieldProperty(interfaces.IAttributeTemplate['default'])
    
class BoolTemplate(AttributeTemplate):
    implements(interfaces.IBoolTemplate)

    default = FieldProperty(interfaces.IBoolTemplate['default'])

class TextTemplate(AttributeTemplate):
    implements(interfaces.ITextTemplate)

    default = FieldProperty(interfaces.ITextTemplate['default'])
    cols = FieldProperty(interfaces.ITextTemplate['cols'])
    rows = FieldProperty(interfaces.ITextTemplate['rows'])

class IntTemplate(AttributeTemplate):
    implements(interfaces.IIntTemplate)

    default = FieldProperty(interfaces.IIntTemplate['default'])
    min = FieldProperty(interfaces.IIntTemplate['min'])
    max = FieldProperty(interfaces.IIntTemplate['max'])

class DatetimeTemplate(AttributeTemplate):
    implements(interfaces.IDatetimeTemplate)

    default = FieldProperty(interfaces.IDatetimeTemplate['default'])

class URITemplate(AttributeTemplate):
    implements(interfaces.IURITemplate)

    default = FieldProperty(interfaces.IURITemplate['default'])
    cols = FieldProperty(interfaces.IURITemplate['cols'])

class RFC822MailAddressTemplate(AttributeTemplate):
    implements(interfaces.IRFC822MailAddressTemplate)

    default = FieldProperty(interfaces.IRFC822MailAddressTemplate['default'])
    cols = FieldProperty(interfaces.IRFC822MailAddressTemplate['cols'])

class ObjectTemplate(AttributeTemplate):
    implements(interfaces.IObjectTemplate)

    default = FieldProperty(interfaces.IObjectTemplate['default'])

