#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.annotation.attribute import AttributeAnnotations
from zope.interface import directlyProvides
from zope.app.zopeappgenerations import getRootFolder
from zope.app.generations.utility import findObjectsProviding

from zw.vogon.question.interfaces import IResponseFolder
from zw.vogon.question.rating.interfaces import IRatingQuestion
from zw.vogon.question.rating.rating import updateResponseSchema

def evolve(context):
    """Change the no_rating option to multiple no_rating options.
    """
    root = getRootFolder(context)
    for rating_question in findObjectsProviding(root, IRatingQuestion):
        no_rating = getattr( rating_question, 'no_rating', None)
        if no_rating is not None:
            setattr( rating_question, 'no_ratings', [no_rating] )
            updateResponseSchema(rating_question, None)
            schema = rating_question.response_schema
            try:
                responses = AttributeAnnotations(rating_question)['zw.vogon.question.responses']
            except KeyError:
                continue
            else:
                for response in responses.values():
                    directlyProvides(response, schema)
                    if response.response == 'no-rating':
                        response.response = no_rating

            
