#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.annotation.attribute import AttributeAnnotations
from zope.interface import directlyProvides
from zope.app.zopeappgenerations import getRootFolder
from zope.app.generations.utility import findObjectsProviding

from zw.vogon.participant.interfaces import IParticipant, IParticipantStates, \
    IParticipants, IParticipantStatus
from zw.vogon.survey.interfaces import ISurvey, ISurveyStatus, ISurveyStates


survey_states = {
    'new_survey': ISurveyStates['blank'],
    'prepare': ISurveyStates['blank'],
    'ready': ISurveyStates['ready'],
    'running': ISurveyStates['running'],
    'finalised': ISurveyStates['finalised']
    }


participant_states = {
    'new_participant': IParticipantStates['ready'],
    'invited': IParticipantStates['active'],
    'failed_to_send': IParticipantStates['failure'],
    'finished_survey': IParticipantStates['finished'],
    'inactive': IParticipantStates['inactive']
    }

def evolve(context):
    """ore.workflow -> zw.status transition
    """
    root = getRootFolder(context)

    # convert the survey status
    for survey in findObjectsProviding(root, ISurvey):
        if 'hurry.workflow.state' in AttributeAnnotations(survey):
            state = AttributeAnnotations(survey)['hurry.workflow.state']
            status = ISurveyStatus(survey)
            try:
                status.mapping['state'] = survey_states[state].__name__
            except KeyError:
                pass

        # Remove old attributes
        try:
            del survey.__annotations__['hurry.workflow.state']
        except:
            pass
        try:
            del survey.__annotations__['hurry.workflow.id']
        except:
            pass

        # convert the participant status
        for participant in IParticipants(survey).values():
            if 'hurry.workflow.state' in AttributeAnnotations(participant):
                state = AttributeAnnotations(participant)['hurry.workflow.state']
                status = IParticipantStatus(participant)
                try:
                    status.mapping['state'] =  participant_states[state].__name__
                except KeyError:
                    pass

            # Remove old attributes
            try:
                del participant.__annotations__['hurry.workflow.state']
            except:
                pass
            try:
                del participant.__annotations__['hurry.workflow.id']
            except:
                pass
