#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.annotation.attribute import AttributeAnnotations
from zope.event import notify
from zope.interface import directlyProvides
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
from zope.app.container.contained import ObjectAddedEvent
from zope.app.zopeappgenerations import getRootFolder
from zope.app.generations.utility import findObjectsProviding

from zw.vogon.portal.interfaces import IPortal
from zw.vogon.settings.interfaces import ISettings
from zw.vogon.settings.settings import SettingsRoot, UpdateSettingsEvent
from zw.vogon.utils.dublincore import updateDCInfos

def evolve(context):
    """evolve to the new settings implementation
    """
    root = getRootFolder(context)

    for portal in findObjectsProviding(root, IPortal):
        sm = portal.getSiteManager()
    
        # 1st create all settings
        if 'settings' not in portal.keys():
            settings = SettingsRoot()
            notify(ObjectCreatedEvent(settings))
            portal['settings'] = settings
            notify(ObjectAddedEvent(settings))
            sm.registerUtility( settings, ISettings )

        notify(UpdateSettingsEvent(settings, portal))

        # 2nd remove old settings
        try:
            old_settings = sm['default']['settings']
        except KeyError:
            pass
        else:
            for r in sm.registeredUtilities():
                if r.component == old_settings:
                    sm.unregisterUtility(r.component, r.provided, r.name)
            del sm['default']['settings']

        # 3rd convert and remove old z3ext.controlpanel relicts in portal
        if 'z3ext.controlpanel.Settings' in AttributeAnnotations(portal):
            data = AttributeAnnotations(portal)['z3ext.controlpanel.Settings']
            for name, item in data.items():
                for fname, value in item.items():
                    setattr(settings[name], fname, value)

            # Remove old attributes
            try:
                del portal.__annotations__['z3ext.controlpanel.Settings']
            except:
                pass
        
        # 4th update portal's dublincore
        updateDCInfos(portal, None)
