#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _


import zope.traversing.api
from zope.schema import Bool
from zope.component import adapts, getMultiAdapter, queryMultiAdapter
from zope.interface import implements, Interface
from zope.proxy import sameProxiedObjects
from zope.security import canAccess
from zope.viewlet import viewlet
from zope.app.folder.interfaces import IRootFolder
from z3c.breadcrumb.browser import Breadcrumbs as BaseBreadcrumbs
from z3c.breadcrumb.browser import GenericBreadcrumb
from z3c.breadcrumb.interfaces import IBreadcrumb

from zw.vogon.layer import IVogonBrowserLayer

class IRootBreadcrumb(Interface):

    isRoot = Bool(title = u"isRoot", default=True)

class RootBreadcrumb(GenericBreadcrumb):
    adapts(IRootFolder, IVogonBrowserLayer)
    implements(IRootBreadcrumb)


    isRoot = True

    def __init__(self, context, request):
        super(RootBreadcrumb, self).__init__(context, request)

    @property
    def activeURL(self):
        view = queryMultiAdapter( (self.context, self.request),
                                  name = u'contents.html' )
        if view is None:
            return False
        return canAccess(view, '__call__')


class Breadcrumbs(BaseBreadcrumbs):
    
    @property
    def crumbs(self):
        request = self.request

        objects = []
        for obj in ( [self.context] +
                     list(zope.traversing.api.getParents(self.context)) ):
            objects.append(obj)
            if sameProxiedObjects(obj, request.getVirtualHostRoot()) or \
                    isinstance(obj, Exception):
                break

        objects.reverse()
        for object in objects:
            info = getMultiAdapter( (object, self.request), IBreadcrumb)
            yield {'name': info.name,
                   'url': info.url,
                   'activeURL': info.activeURL,
                   'isRoot': IRootFolder.providedBy(object) }


class BreadcrumbsViewlet(viewlet.simple, viewlet.ViewletBase):

    def update(self):
        self.breadcrumbs = Breadcrumbs(self.context, self.request)

