#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
""" 
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import email.Charset
email.Charset.add_charset('utf-8', email.Charset.SHORTEST, None, None)
import base64
from datetime import datetime
import pytz
import email

from email.MIMEText import MIMEText
from email.MIMEMessage import MIMEMessage
from email.MIMEMultipart import MIMEMultipart

# Python 2.5:
#from email.mime.text import MIMEText
#from email.mime.message import MIMEMessage
#from email.mime.multipart import MIMEMultipart

from zope.component import adapter, getUtility
from zope.sendmail.interfaces import IMailDelivery

from zw.mail.incoming.interfaces import IIncomingEmailEvent, \
    IIncomingEmailFailureEvent

from zw.vogon.app.interfaces import IApplicationSettings
from zw.vogon.i18n import umlaut_normalizer
from zw.vogon.portal.interfaces import IPortal
from zw.vogon.participant.participant import participantInvitationFailed, \
    participantRemindingFailed
from zw.vogon.principal.interfaces import IUserFolder
from zw.vogon.principal.principal import disablePrincipal
from zw.vogon.mail.settings import IMailSettings



@adapter(IIncomingEmailEvent)
def processMessage(event):
    msg = event.message
    inbox = event.inbox
    root = event.root
    to = msg['To']
    mailer = getUtility(IMailDelivery, 'zw.vogon')

    forward = False

    portal = None
    for key, _portal in root.items():
        if not IPortal.providedBy(_portal):
            continue
        mail_settings = getUtility(IMailSettings, context=_portal)
        if mail_settings.email_from_address in to:
            portal = _portal
            break

    if portal is None:
        

        if msg.get('X-Vogon-Admin', None) == 'Application':
            print "It seems we have a mail loop. Maybe the applications admin address is wrong."
            
        else:
            app_settings = getUtility(IApplicationSettings, context=root)

            mail_text = u"""Dear administrator,

the attached email reached the Vogon application, but 
it cannot be assigned to any portal.

Best regards,
Your Vogon application"""

            message = MIMEMultipart()
            message.attach( MIMEText( mail_text.encode('utf-8'),
                                      'plain', 'utf-8'  ) )

            message['Subject'] = '[Vogon] %s' % msg['subject']
            message['Date'] = datetime.now(pytz.utc).strftime( '%a, %d %b %Y %H:%M:%S %z')
            message['From'] = "Vogon application <%s>" % app_settings.sender_address
            message['To'] = app_settings.admin_address
            
            # To let us find failures
            message['X-Vogon-Forward'] = 'Application'

            message.attach( MIMEMessage(msg) )

            mailer.send( app_settings.sender_address, 
                         ( app_settings.admin_address, ), message.as_string() )
            
    elif IIncomingEmailFailureEvent.providedBy(event):
        # Decide if msg is a principal or participant one
        for part in msg.walk():
            if 'X-Vogon-PID' in part:
                if disablePrincipal(portal, part['X-Vogon-PID'], 
                                    event.delivery_report) is not None:
                    # We couldn't disable principal
                    forward = True

            elif 'X-Vogon-Invite' in part:
                if participantInvitationFailed(portal, part['X-Vogon-Invite'],
                                               event.delivery_report) is not None:
                    forward = True
                
            elif 'X-Vogon-Reminder' in part:
                if participantRemindingFailed(portal, part['X-Vogon-Reminder'], 
                                              event.delivery_report) is not None:
                    forward = True
                

    else: 
        forward = True
    
    if forward:
        message = MIMEMultipart()

        if msg.get('X-Vogon-Forward', None) == 'Portal':
            print "It seems we have a mail loop. Maybe the portal admin address is wrong. Forwarding to admin"
            app_settings = getUtility(IApplicationSettings, context=root)
            to_addresses = (app_settings.admin_address, )

            # To let us find failures
            message['X-Vogon-Forward'] = 'Application'

            mail_text = u"""Dear administrator,

the attached email reached the Vogon application, but 
it cannot be forwarded to any portal administrator.

Best regards,
Your Vogon application"""
            
        else:
            users = getUtility(IUserFolder, name='users', context=portal)
            to_addresses = []
            for user_id in mail_settings.portals_mail_admins:
                try:
                    principal = users[user_id[len(users.prefix):]]
                except KeyError:
                    continue
                else:
                    to_addresses.append(principal.email)
            to_addresses = tuple(to_addresses)
            message['X-Vogon-Forward'] = 'Portal'
            mail_text = u"""Dear portal administrator,

the attached email reached the Vogon portal of '%s'.

Yours sincerely,
Your Vogon application""" % portal.title

        from_address = mail_settings.email_from_address

        message.attach( MIMEText( mail_text.encode('utf-8'),
                                  'plain', 'utf-8'  ) )
        
        message['Subject'] = '[Vogon] %s' % msg['subject']
        message['Date'] = datetime.now(pytz.utc).strftime( '%a, %d %b %Y %H:%M:%S %z')
        message['From'] = "Portal %s <%s>" % (umlaut_normalizer( portal.title ), from_address )
        message['To'] = ', '.join(to_addresses)
        
        message.attach( MIMEMessage(msg) )
        
        mailer.send( from_address, to_addresses, message.as_string() )

        
        
    inbox.delete(msg)
