#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
""" 
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.interface import Interface

from z3c.form import field
from z3c.form.browser.select import CollectionSelectFieldWidget
from z3c.schema.email import RFC822MailAddress

from zw.vogon.settings.browser.settings import EditSettings

class IMailSettings(Interface):
    """Mail settings.
    """
    
    email_from_address = RFC822MailAddress(
        title = _( u'label-mail.settings.IMailSettings.email_from_address',
                   u"Portal's sender address" ),
        description = _( u'help-mail.settings.IMailSettings.email_from_address',
                         u"Enter the email address to be used to send emails "
                         u"from this site." ),
        required = True )
    

    portals_mail_admins = schema.List(
        title = _( u'label-mail.settings.IMailSettings.portals_mail_admins',
                   u"Portal's mail administrators" ),
        description = _( u'help-mail.settings.IMailSettings.portal_mail_admins',
                         u"Choose the portal users, which should receive ",
                         u"eMails sent to the upper address." ),
        default = [],
        value_type = schema.Choice(
            title = u"portal member",
            vocabulary = u"zw.vogon.principals" ),
        )

class EditMailSettings(EditSettings):
    """Edit mail settings
    """
    
    fields = field.Fields(IMailSettings)
    fields['portals_mail_admins'].widgetFactory = CollectionSelectFieldWidget
    

