#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""

from zw.vogon.i18n import MessageFactory as _

from base64 import urlsafe_b64decode

from zope import i18n, schema
from zope.cachedescriptors.property import Lazy
from zope.component import getAdapter, getMultiAdapter
from zope.security.management import checkPermission
from zope.traversing.api import getParent
from zope.app.renderer.plaintext import PlainTextSourceFactory, \
    PlainTextToHTMLRenderer

from z3c.form import button, field, form
from z3c.form.interfaces import IDataConverter
from z3c.schema.email.interfaces import NotValidRFC822MailAdress

from zc.table import column

from zw.contentsform.column import FieldColumn, Columns, FormActionsColumn
from zw.contentsform.contentsform import ContentsForm
from zw.contentsform.cutcopypaste import CutCopyPasteMixin
from zw.contentsform.interfaces import IEditContentRow

from zw.status.interfaces import InvalidTransactionError

from zw.vogon.participant.browser.emails_widget import EmailsFieldWidget
from zw.vogon.participant.interfaces import IParticipant, IParticipants, \
    IParticipantStatus
from zw.vogon.participant.participant import Participant
from zw.vogon.survey.interfaces import ISurveyStatus

class StatusColumn(column.Column):

    title = u"Status"
    name = u'status'

    def __init__(self):
        pass

    def renderCell(self, item, formatter):
        # TODO: Check permissions !!!

        ob = item.getContent()
        obStatus = IParticipantStatus(ob)
        # Quick hack to obscurify if a participant has already
        # filled in the form.
        status = obStatus.state.title
        if not checkPermission(u'zw.vogon.viewRealParticipantStatus', ob) and \
                obStatus.state in ( 
            obStatus.states['active'],
            obStatus.states['active_started_fillin'],
            obStatus.states['inactive'],
            obStatus.states['inactive_started_fillin'],
            obStatus.states['finished'] ):
            status = 'Ok'
            
        return i18n.translate(
            status, context = formatter.request )
                          
        
class RemarksColumn(column.Column):

    title = u"Remarks"
    name = u'remarks'

    def __init__(self):
        pass

    def renderCell(self, item, formatter):
        obj = item.getContent()
        # render remark
        if obj.remark is None:
            return u''
        source = PlainTextSourceFactory(obj.remark)
        renderer = PlainTextToHTMLRenderer(source, formatter.request)
        return renderer.render()

class ManageParticipantsForm(CutCopyPasteMixin, ContentsForm):
    form.extends(CutCopyPasteMixin)
    
    canCut = False
    canCopy = False # Maybe later
    canPaste = False
    canDelete = True
    

    @property
    def label(self):
        return _( u'label-participant.browser.participants.ManageParticipantsForm.label',
                  u"${title} - Participants",
                  mapping = { 'title': self.context.title } )

    cssClasses = {'table': 'listing' }

    @Lazy
    def container(self):
        return getAdapter(self.context, IParticipants)
    
    def getContents(self):
        for p in self.container.values():
            status = IParticipantStatus(p)
            if status.state != status.states['failure']:
                yield p

    def _getColumns(self):
        return Columns(
            FieldColumn( IParticipant['email_address'] ),
            StatusColumn(),
            RemarksColumn()
            )

    @Lazy
    def survey_status(self):
        return ISurveyStatus(self.context)

    def update(self):
        super(ManageParticipantsForm, self).update()

        failure_form = FailureParticipantsForm(self.context, self.request, self)
        failure_form.prefix = self.prefix+'failure_form.'
        failure_form.update()
        self.failure_form = failure_form

        add_form = AddParticipantsForm(self.container, self.request, self)
        add_form.prefix = self.prefix+'add_form.'
        add_form.update()
        self.add_form = add_form


    @button.buttonAndHandler( _( u'Invite' ), name='invite',
                              condition=lambda form: form.getContents() and \
                                  form.survey_status.state == form.survey_status.states['running'] )
    def handleStartSurveys(self, action):
        started = []
        invalid_status = []
        # collect selected participants
        for id in self.request.form.get(self.sessionKey+u'.ids', []):
            ob = self.container[id]
            status = IParticipantStatus(ob)
            try:
                status.startTransaction(status.invite)
            except InvalidTransactionError: 
                invalid_status.append(ob)
            else:
                started.append(ob)


class FailureParticipantsForm(CutCopyPasteMixin, ContentsForm):
    form.extends(CutCopyPasteMixin)
    
    canCut = False
    canCopy = False # Maybe later
    canPaste = False
    canDelete = True

    cssClasses = {'table': 'listing' }

    def __init__(self, context, request, parentForm):
        super(FailureParticipantsForm, self).__init__(context, request)
        self.__parent__ = self.parentForm = parentForm


    @property
    def haveFailures(self):
        try:
            self.getContents().next()
        except StopIteration:
            return False
        return True
    
    @Lazy
    def container(self):
        return getAdapter(self.context, IParticipants)
    
    def getContents(self):
        for p in self.container.values():
            status = IParticipantStatus(p)
            if status.state == status.states['failure']:
                yield p

    def _getColumns(self):
        return Columns(
            FieldColumn( IParticipant['email_address'] ),
            StatusColumn(),
            RemarksColumn(),
            #FormActionsColumn(),
            )

    #def getRows(self):
    #    for item in self.getContents():
    #        # All item are to edit
    #        row = getMultiAdapter( (self, item), IEditContentRow)
    #        row.update()
    #        yield row


class AddParticipantsForm(form.AddForm):


    fields = field.Fields(
        schema.List(__name__='email_addresses',
                    title = _( 
                u'label-survey.participant.browser.IAddingSchema.email_addresses',
                u"Email adresses" ),
                    description = _(
                u'help-survey.participant.browser.IAddingSchema.email_addresses',
                u"Enter email addresses to add - one per line." ),
                    required = False,
                    value_type = schema.TextLine(
                title = _( u'label-survey.participant.browser.IAddingSchema.email_addresses.value_type',
                           u"Email address" ) )
                    )
        )
    fields['email_addresses'].widgetFactory = EmailsFieldWidget

    def __init__(self, context, request, parentForm):
        super(AddParticipantsForm, self).__init__(context, request)
        self.__parent__ = self.parentForm = parentForm
        # self.session = parentForm.session


    def update(self):
        super(AddParticipantsForm, self).update()
        failed = self.request.get('failed', None)
        if failed is not None:
            self.status =  _( u"The following email addresses aren't correct." )
            self.widgets['email_addresses'].value = urlsafe_b64decode(
                str(failed) ).decode('utf-8')

    def createAndAdd(self, data):
        addresses = data.get('email_addresses')
        if addresses is None:
            addresses = []
        
        addresses = set(addresses).difference([u''])
        add_addresses = addresses.copy()

        for address in addresses:
            try:
                IParticipant['email_address'].validate(address)
            except NotValidRFC822MailAdress:
                continue
            else:
                if self.addParticipant(address) is not None:
                    add_addresses.discard(address)

        if add_addresses:
            self.status = _( u"The following email addresses aren't correct." )

        converter = IDataConverter(self.widgets['email_addresses'])
        self.widgets['email_addresses'].value = converter.toWidgetValue(
            list(add_addresses) )



    def addParticipant(self, address):
        folder = self.context
        if address not in folder:
            obj = Participant( email_address=address )
            folder[address] = obj
            return obj
        return None

