#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _
from zope.component import adapter, getUtility
from zope.component.interfaces import ObjectEvent
from zope.event import notify
from zope.interface import implements, implementsOnly
from zope.lifecycleevent import ObjectCreatedEvent
from zope.sendmail.interfaces import IMailDelivery
from zope.app.catalog.field import FieldIndex

from zw.status.interfaces import IStateChangedEvent

from zw.vogon.principal.interfaces import INewPortalPAUEvent
from zw.vogon.participant.interfaces import IHasParticipants, IParticipants, \
    IParticipantEvent, IParticipantActiveEvent, IParticipantInactiveEvent, \
    IParticipantFinishedEvent, IParticipantResetEvent, IParticipantStatus, \
    IParticipant
from zw.vogon.portal.interfaces import INewCatalogEvent
from zw.vogon.survey.survey import SurveyStatus
from zw.vogon.survey.interfaces import ISurveyStates, ISurveyFinishedEvent

class ParticipantEvent(ObjectEvent):
    implements(IParticipantEvent)

    def __init__(self, obj, survey):
        super(ParticipantEvent, self).__init__(obj)
        self.survey = survey

class ParticipantActiveEvent(ParticipantEvent):
    implements(IParticipantActiveEvent)

class ParticipantInactiveEvent(ParticipantEvent):
    implements(IParticipantInactiveEvent)

class ParticipantFinishedEvent(ParticipantEvent):
    implements(IParticipantFinishedEvent)

class ParticipantResetEvent(ParticipantEvent):
    implements(IParticipantResetEvent)


@adapter(INewPortalPAUEvent)
def setupPauForParticipants(event):
    pau = event.object

    pau.credentialsPlugins = [ u'Participant Credentials' ] + \
        list(pau.credentialsPlugins)
    pau.authenticatorPlugins = [ u'Participant Authenticator' ] + \
        list(pau.authenticatorPlugins)


@adapter(IHasParticipants, IStateChangedEvent)
def handleStateChange(survey, event):
    pFolder = IParticipants(survey)
    mailer = getUtility(IMailDelivery, name=u'zw.vogon')

    if event.newState == ISurveyStates['ready'] and \
            event.transaction == SurveyStatus.prepare:
        pass
    elif event.newState == ISurveyStates['running']:
        if event.transaction == SurveyStatus.start_survey:
            # start transaction for each participant
            for participant in pFolder.values():
                status = IParticipantStatus(participant)
                if status.invite in status.getTransactions():
                    status.startTransaction(status.invite)
                    
        elif event.transaction == SurveyStatus.remind:
            for participant in pFolder.values():
                status = IParticipantStatus(participant)
                if status.remind in status.getTransactions():
                    status.startTransaction(status.remind)

        elif event.transaction == SurveyStatus.restart:
            for participant in pFolder.values():
                status = IParticipantStatus(participant)
                if status.reactivate in status.getTransactions():
                    status.startTransaction(status.reactivate)
        
    elif event.newState == ISurveyStates['finalised']:
        if event.transaction == SurveyStatus.stop_survey:
            # Deactivate all pariticipants
            for participant in pFolder.values():
                status = IParticipantStatus(participant)
                if status.deactivate in status.getTransactions():
                    status.startTransaction(status.deactivate)

    elif event.transaction == SurveyStatus.reset:
        for participant in pFolder.values():
            status = IParticipantStatus(participant)
            if status.reset in status.getTransactions():
                status.startTransaction(status.reset)

    elif event.newState == ISurveyStates['archived']:
        # NIY
        pass
    
    
@adapter(IHasParticipants, ISurveyFinishedEvent)
def deactivateParticipant(survey, event):
    participant = event.principal.participant
    status = IParticipantStatus(participant)
    status.startTransaction(status.finish)


@adapter(INewCatalogEvent)
def setUpCatalog(event):
    catalog = event.object

    # For lookup up participants
    catalog['participant_uid'] = FieldIndex('uid', IParticipant, False)
    #catalog['participant_survey'] = FieldIndex('getSurveyId', IParticipant, True)


