#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#
#   Copyright (c) 200
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import random

from zope.annotation.factory import factory
from zope.cachedescriptors.property import Lazy
from zope.component import adapts, adapter
from zope.interface import implements, implementer
from zope.traversing.api import getParent
from zope.app.authentication.interfaces import IPrincipalInfo
from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import uncontained
from zope.app.container.interfaces import IObjectRemovedEvent

from zw.vogon.survey.interfaces import ISurveyStatus, ISurveyStates
from zw.vogon.participant.interfaces import IParticipants, IParticipant, \
    IParticipantsManagement, IParticipantContainer, \
    IParticipantStatus, IParticipantStates, \
    IParticipantEvent
from zw.vogon.participant.interfaces import IHasParticipants

class Participants(BTreeContainer):
    implements(IParticipants, IParticipantsManagement, IParticipantContainer)
    adapts(IHasParticipants)

    _randrange = random.randrange
    maxint = int(2**32-1)

    def generateId(self):
        """Generate an id which is not yet taken.
        """
        while True:
            uid = '%x' % self._randrange(0, self.maxint)
            if uid not in [ p.uid for p in self.values() ]:
                return uid

    def getParticipantByUid(self, uid):
        for item in self.values():
            if item.uid == uid:
                return item
        return

    def authenticateParticipant(self, uid):
        # XXXX
        survey = getParent(self)
        if ISurveyStatus(survey).state != ISurveyStates['running']:
            return
        participant = None
        for item in self.values():
            if item.uid == uid:
                status = IParticipantStatus(item)
                if status.state == IParticipantStates['active']:
                    status.startTransaction(status.startFillin)
                if status.state == IParticipantStates['active_started_fillin']:
                    participant = item
                    break
            
        if participant is None:
            return

        return IPrincipalInfo(participant)

    @Lazy
    def numberOfFinished(self):
        finished = [ 1 for p in self.values() \
                         if IParticipantStatus(p).state in (
                IParticipantStates['finished'], ) ]
        return len(finished)

    @Lazy
    def numberOfFailures(self):
        return len([ 1 for p in self.values() \
                         if IParticipantStatus(p).state in (
                IParticipantStates['failure'], ) ])


participantsFactory = factory(Participants, 'zw.vogon.participants')

@adapter(IParticipant, IParticipantEvent)
def updateNumberOfParticipants(participant, event):
    folder = getParent(participant)
    try:
        del folder.numberOfFinished
        del folder.numberOfFailures
    except AttributeError:
        pass

@adapter(IHasParticipants, IObjectRemovedEvent)
def removeParticipants(survey, event):
    folder = IParticipants(survey)
    for name, item in folder.items():
        del folder[name]
