#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapts
from zope.event import notify
from zope.interface import implements
from zope.publisher.interfaces import IRequest
from zope.traversing.api import getParent
from zope.app.authentication.interfaces import AuthenticatedPrincipalCreated, \
    FoundPrincipalCreated
from zope.app.authentication.principalfolder import PrincipalInfo, Principal, \
    AuthenticatedPrincipalFactory, FoundPrincipalFactory

from zw.vogon.survey.interfaces import ISurvey
from zw.vogon.participant.interfaces import IParticipant, IHasParticipants, \
    IParticipantPrincipal, IParticipantPrincipalInfo

class ParticipantPrincipalInfo(PrincipalInfo):
    implements(IParticipantPrincipalInfo)
    adapts(IParticipant)

    survey = None
    participant = None

    def __init__(self, participant):
        super(ParticipantPrincipalInfo, self).__init__(
            id = participant.uid,
            login = participant.email_address,
            title = _( u'Participant' ),
            description = u'' )

        self.survey = participant.survey
        self.participant = participant
        
class ParticipantPrincipal(Principal):
    implements(IParticipantPrincipal)
    
    survey = None
    participant = None

    def __init__(self, id, title=u'', description=u'', survey=None, 
                 participant=None):
        super(ParticipantPrincipal, self).__init__(id, title, description)
        self.survey = survey
        self.participant = participant
        

    def __repr__(self):
        return 'ParticipantPrincipal(%r of %s)' % (self.id, self.survey)


class ParticipantPrincipalFactory(AuthenticatedPrincipalFactory):
    adapts(IParticipantPrincipalInfo, IRequest)

    def __call__(self, authentication):
        principal = ParticipantPrincipal(authentication.prefix + self.info.id,
                                         self.info.title,
                                         self.info.description,
                                         self.info.survey,
                                         self.info.participant)
        notify(AuthenticatedPrincipalCreated(
                authentication, principal, self.info, self.request))
        return principal


class FoundParticipantPrincipalFactory(FoundPrincipalFactory):
    adapts(IParticipantPrincipalInfo)

    
    def __call__(self, authentication):
        principal = ParticipantPrincipal(authentication.prefix + self.info.id,
                                         self.info.title,
                                         self.info.description,
                                         self.info.survey,
                                         self.info.participant)
        notify(FoundPrincipalCreated(authentication,
                                     principal, self.info))
        return principal
