#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapter, getUtility
from zope.event import notify
from zope.interface import alsoProvides, implements
from zope.lifecycleevent import ObjectCreatedEvent
from zope.session.http import ICookieClientIdManager, CookieClientIdManager
from zope.session.interfaces import ISessionDataContainer
from zope.session.session import PersistentSessionDataContainer

from zw.vogon.portal.interfaces import INewPortalEvent, INewCatalogEvent

class NewPortalEvent(ObjectCreatedEvent):
    implements(INewPortalEvent)


@adapter(INewPortalEvent)
def setUpClientIdAndSessionDataContainer(event):
    sm = event.object.getSiteManager()

    clientids = CookieClientIdManager()
    clientids.namespace = u'vogon.'+event.object.__name__
    clientids.cookieLifetime = 3600
    sm['default']['clientids'] = clientids
    sm.registerUtility(sm['default']['clientids'], ICookieClientIdManager)

    session_data = PersistentSessionDataContainer()
    session_data.timeout = 3600
    session_data.resolution = 5
    sm['default']['session_data'] = session_data
    sm.registerUtility(sm['default']['session_data'], ISessionDataContainer)

from zope.app.intid import IntIds
from zope.app.intid.interfaces import IIntIds

from zope.app.catalog.catalog import Catalog
from zope.app.catalog.interfaces import ICatalog

class NewCatalogEvent(ObjectCreatedEvent):
    implements(INewCatalogEvent)


@adapter(INewPortalEvent)
def setUpIntIds(event):
    sm = event.object.getSiteManager()

    intids = IntIds()
    sm['default']['intids'] = intids
    notify(ObjectCreatedEvent(intids))
    sm.registerUtility(intids, IIntIds)

@adapter(INewPortalEvent)
def setUpCatalog(event):
    sm = event.object.getSiteManager()

    catalog = Catalog()
    sm['default']['catalog'] = catalog
    notify(ObjectCreatedEvent(catalog))
    sm.registerUtility(catalog, ICatalog, 'catalog')

    notify(NewCatalogEvent(catalog))
    
from zope.error.error import ErrorReportingUtility
from zope.error.interfaces import IErrorReportingUtility

@adapter(INewPortalEvent)
def setUpErrorReportingUtility(event):
    sm = event.object.getSiteManager()

    eru = ErrorReportingUtility()
    sm['default']['errors'] = eru
    notify(ObjectCreatedEvent(eru))
    sm.registerUtility(eru, IErrorReportingUtility)

