#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <giesen@zaehlwerk.net>            #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapter, adapts, getUtility
from zope.component.interfaces import ComponentLookupError
from zope.event import notify
from zope.interface import alsoProvides, implements
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.api import getParents

from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import ObjectAddedEvent
from zope.app.container.interfaces import IObjectAddedEvent
from zope.app.component.site import SiteManagerContainer, LocalSiteManager

from zw.vogon.portal.events import NewPortalEvent
from zw.vogon.portal.interfaces import IPortal, IPortalContainer
from zw.vogon.portal.settings import IPortalSettings

class Portal(BTreeContainer, SiteManagerContainer):
    implements(IPortal, IPortalContainer)

    @property
    def title(self):
        try:
            return getUtility(IPortalSettings, context=self).title
        except ComponentLookupError:
            return u''
    
    def setSiteManager(self, sm):
        super(Portal, self).setSiteManager(sm)
        notify(NewPortalEvent(self))

def getPortal(context):
    for item in [context] + getParents(context):
        if IPortal.providedBy(item):
            return item
    return None
alsoProvides(getPortal, IPortal)

@adapter(IPortal, IObjectAddedEvent)
def setSiteManagerWhenAdded(site, event):
    site.setSiteManager(LocalSiteManager(site))

