#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import adapts, createObject, getUtility
from zope.interface import implements, Interface
from zope.security.interfaces import IPrincipal
from zope.traversing.api import getParent, getName
from zope.app.authentication.interfaces import IPrincipalInfo
from zope.app.authentication.principalfolder import IInternalPrincipal
from zope.app.authentication.groupfolder import IGroupFolder
from zope.app.security.interfaces import IAuthentication

from z3c.form import button, form
from z3c.form.interfaces import DISPLAY_MODE

from zw.contentsform.adding import AddingFormMixin
from zw.contentsform.column import FieldColumn, ContentNameColumn, Columns, \
    FormActionsColumn
from zw.contentsform.contentsform import ContentsForm
from zw.contentsform.cutcopypaste import CutCopyPasteMixin
from zw.contentsform.row import AddContentRow, EditContentRow

from zw.vogon.principal.interfaces import IUserFolder, IMailPassword, \
    IInternalUser, IPrincipalStatus

class IManagePrincipals(Interface):
    """Manage users.
    """

class ManagePrincipals(AddingFormMixin, CutCopyPasteMixin, ContentsForm):
    implements(IManagePrincipals)
    form.extends(AddingFormMixin, CutCopyPasteMixin)

    label = _( u'label-principal.browser.manage.ManagePrincipals.label',
               u"User management" )

    cssClasses = {'table': 'listing' }
    canCut = False
    canCopy = False
    canPaste = False

    @Lazy
    def container(self):
        principal_folder = getUtility(IUserFolder, name=u'users')
        return principal_folder

    #def getContents(self):
    #    #pau = getUtility(IAuthentication)
    #    for item in self.container.values():
    #        yield self.container.principalInfo(
    #            self.container.getIdByLogin(item.login) )

    def _getColumns(self):
        return Columns(
            FieldColumn( IInternalPrincipal['login'], readonly=True ),
            FieldColumn( IInternalPrincipal['title'], title=_( u'Fullname' ) ),
            FieldColumn( IMailPassword['email'] ),
            FieldColumn( IPrincipalStatus['active'], readonly=True ),
            FieldColumn( IPrincipalStatus['remark'], readonly=True ),
            FormActionsColumn( title = _( u'Reset password' ) )
            )


    @button.buttonAndHandler( _( u'Add user' ), name='add_user',
                              condition = lambda form: \
                                  form.session.get('add') is None )
    def handleAddUser(self, action):
        self.session['add'] = u'zw.vogon.principal.InternalUser'
        self.request.response.redirect(self.request.getURL())


class AddPrincipalRow(AddContentRow):
    adapts(IManagePrincipals, IUserFolder)

    def create(self, data):
        obj = createObject(self.factoryname,
                           login=u'', password=u'', title=u'',
                           passwordManagerName='SHA1')
        changes = form.applyChanges(self, obj, data)
        if IMailPassword.providedBy(obj):
            obj.resetPassword()
        return obj

    def add(self, content):
        content = super(AddPrincipalRow, self).add(content, content.login)
        pau = getUtility(IAuthentication)
        auth = getParent(content)
        id = auth.prefix + getName(content)
        for name, auth_plugin in pau.getAuthenticatorPlugins():
            if IGroupFolder.providedBy(auth_plugin): 
                members_group = auth_plugin.get('members')
                if members_group is not None:
                    members_group.setPrincipals(
                        list(members_group.principals) + [id] )
        return content


class DisplayPrincipalRow(EditContentRow):
    adapts(IManagePrincipals, IInternalUser)

    mode = DISPLAY_MODE
    ignoreRequest = True
    

    @button.buttonAndHandler( _( u"Reset" ), name = 'reset' )
    def handleReset(self, action):
        if IMailPassword.providedBy(self.context):
            self.context.resetPassword()
