#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.cachedescriptors.property import Lazy
from zope.component import adapts, createObject, getUtilitiesFor
from zope.interface import implements, Interface, invariant, Invalid
from zope.security.interfaces import IPrincipal
from zope.app.security.interfaces import IAuthentication
from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile

from z3c.form import button, field
from z3c.form.interfaces import HIDDEN_MODE
from z3c.formui import form

from zw.vogon.principal.interfaces import IUserFolder, IMailPassword, IInternalUser



class IPasswordSchema(Interface):
    """The subscription form."""

    id = schema.TextLine( title = u'id', required=False )

    password = schema.Password(
        title=_(u'Password'),
        description=_(u'The new password.'),
        required=True)

    confirm = schema.Password(
        title=_(u'Confirm password'),
        description=_(u'The new password.'),
        required=True)
    
    @invariant
    def confirmPassword(task):
        if task.password != task.confirm:
            raise Invalid(
                _("Password doesn't compare with password confirmation."))
        if len(task.password) < 6:
            raise Invalid(
                _("Password must be at least 6 characters long."))

class PasswordForm(form.Form):

    ignoreContext = True
    ignoreReadonly = True
    
    _finishedChange = False

    label = _( u"Change the password" )
    formErrorsMessage = _( u"Invalid password" )
    fields = field.Fields(IPasswordSchema)

    password_changed_template = ViewPageTemplateFile('password_changed.pt')

    def update(self):
        super(PasswordForm, self).update()
        self.widgets['id'].name = 'id'
        self.widgets['id'].mode = HIDDEN_MODE
        self.widgets['id'].value = self.request.get('id', u'')

    def getContent(self):
        for name, user_folder in getUtilitiesFor(IUserFolder):
            id = self.request.principal.id
            if id.startswith(user_folder.prefix):
                id = id[len(user_folder.prefix):]
                if id in user_folder:
                    return user_folder[id]
        return None

    @button.buttonAndHandler( _( u'Change' ), name = 'change' )
    def handleChange(self, action):
        data, errors = self.extractData()
        content = self.getContent()
        if errors:
            self.status = self.formErrorsMessage
            return
        
        password = data.get('password')
        content.setPassword(password)
        content.active = True
        content.remark = u''
        self._finishedChange = True
        self._pw = password
        
    def render(self):
        if self._finishedChange:
            content = self.getContent()
            return self.password_changed_template(action=self.request.getURL(1),
                                                  login=content.login, 
                                                  password=self._pw)
        return super(PasswordForm, self).render()
        
