#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import getUtility
from zope.interface import implements
from zope.app.pagetemplate import ViewPageTemplateFile
from zope.app.publisher.interfaces.http import ILogin
from zope.app.security.browser import auth
from zope.app.security.interfaces import IAuthentication
from zope.app.security.interfaces import IUnauthenticatedPrincipal
from zope.app.security.interfaces import ILogout, ILogoutSupported
from z3c.pagelet.browser import BrowserPagelet

class HTTPAuthenticationLogin(BrowserPagelet, auth.HTTPAuthenticationLogin):
    confirmation = ViewPageTemplateFile('login.pt')
    failed = ViewPageTemplateFile('login_failed.pt')

    def render(self):
        nextURL = self.request.get('nextURL', None)
        return self.login(nextURL)

class HTTPBasicAuthenticationLogin(BrowserPagelet, 
                                   auth.HTTPBasicAuthenticationLogin):
    confirmation = ViewPageTemplateFile('login.pt')
    failed = ViewPageTemplateFile('login_failed.pt')

    def render(self):
        nextURL = self.request.get('nextURL', None)
        return self.login(nextURL)

class HTTPAuthenticationLogout(BrowserPagelet):
    implements(ILogout)

    template = lambda *args, **kw: u''
    confirmation = ViewPageTemplateFile('logout.pt')
    redirect = ViewPageTemplateFile('redirect.pt')

    def update(self):
        nextURL = self.request.get('nextURL', None)
        if not IUnauthenticatedPrincipal.providedBy(self.request.principal):
            auth = getUtility(IAuthentication)
            ILogout(auth).logout(self.request)
            if nextURL:
                self.template = self.redirect
            return
        if nextURL is None:
            self.template = self.confirmation
        else:
            self.request.response.redirect(nextURL)


class LoginForm(BrowserPagelet):
    template = ViewPageTemplateFile('loginform.pt')
