#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapter, createObject
from zope.event import notify
from zope.interface import implements
from zope.lifecycleevent import ObjectCreatedEvent
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.app.principalannotation.interfaces import IPrincipalAnnotationUtility
from zope.app.authentication.authentication import PluggableAuthentication
from zope.app.authentication.groupfolder import GroupFolder, GroupInformation,\
    IGroupFolder
from zope.app.container.contained import ObjectAddedEvent
from zope.app.security.interfaces import IAuthentication

from zw.vogon.portal.interfaces import INewPortalEvent
from zw.vogon.principal.interfaces import IUserFolder, INewPortalPAUEvent
from zw.vogon.principal.folder import UserFolder

@adapter(INewPortalEvent)
def setUpPluggableAuthenticationUtility(event):
    sm = event.object.getSiteManager()

    pau = PluggableAuthentication()
    notify(ObjectCreatedEvent(pau))
    sm['default']['PluggableAuthentication'] = pau
    notify(ObjectAddedEvent(pau))

    # Register only for root folder ??? No.
    sm.registerUtility(pau, IAuthentication)

    # Add folders for users and groups
    users = UserFolder( prefix=u'users_' )
    notify(ObjectCreatedEvent(users))
    pau['users'] = users
    notify(ObjectAddedEvent(users))
    sm.registerUtility(users, IUserFolder, name=u"users")


    groups = GroupFolder( prefix=u'groups_' )
    notify(ObjectCreatedEvent(groups))
    pau['groups'] = groups
    notify(ObjectAddedEvent(groups))
    #sm.registerUtility(groups, IGroupFolder, name=u"groups")

    pau.authenticatorPlugins = list(pau.authenticatorPlugins) + \
        [ u'users', u'groups' ]
    pau.credentialsPlugins = list(pau.credentialsPlugins) + \
        [ u'No Challenge if Authenticated', u'URL Login Credentials',
          u'Session Credentials' ]

    notify(NewPortalPAUEvent(pau))
    
    role_manager = IPrincipalRoleManager(event.object)

    # Add members group
    member_group = GroupInformation( title = 'Portal members' )
    notify(ObjectCreatedEvent(member_group))
    groups['members'] = member_group
    notify(ObjectAddedEvent(member_group))
    # Wire zw.vogon.PortalUser role
    role_manager.assignRoleToPrincipal( u'zw.vogon.PortalUser',
                                        pau.prefix + groups.prefix + \
                                            'members' )
    role_manager.assignRoleToPrincipal( u'zw.vogon.PortalMember',
                                        pau.prefix + groups.prefix + \
                                            'members' )

    # Add manager group
    manager_group = GroupInformation( title = 'Managers' )
    notify(ObjectCreatedEvent(manager_group))
    groups['managers'] = manager_group
    notify(ObjectAddedEvent(manager_group))
    # Wire zw.vogon.PortalManager role
    role_manager.assignRoleToPrincipal( u'zw.vogon.PortalManager',
                                        pau.prefix + groups.prefix + \
                                            'managers' )


@adapter(INewPortalEvent)
def setUpPrincipalAnnotationUtility(event):
    sm = event.object.getSiteManager()

    pannu = createObject(u'zope.app.PrincipalAnnotationUtility')
    notify(ObjectCreatedEvent(pannu))
    sm['default']['PrincipalAnnotation'] = pannu
    notify(ObjectAddedEvent(pannu))

    sm.registerUtility(pannu, IPrincipalAnnotationUtility)


class NewPortalPAUEvent(ObjectCreatedEvent):
    implements(INewPortalPAUEvent)

