#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.interface import Interface
from zope.lifecycleevent.interfaces import IObjectCreatedEvent
from zope.app.container.interfaces import IContained, IContainer
from zope.app.container.constraints import containers, contains
from z3c.schema.email import RFC822MailAddress


class IPrincipalStatus(Interface):
    """
    """

    active = schema.Bool(
        title = _( u'label-principal.IPrincipalStatus.active',
                   u"Active" ),
        description = _( u'help-principal.IPrincipalStatus.active',
                         u"An inactive principal is unable to log in." ),
        required = True,
        default = False )

    remark = schema.TextLine(
        title = _( u'label-principal.IPrincipalStatus.remark',
                   u"Remark" ),
        description = _( u'help-principal.IPrincipalStatus.remark',
                         u"An optional short remark about the recent status." ),
        required = False )
    

class IMailPassword(Interface):
    """Extends a principal by the capability to send a mail with a link
    to renew the password.
    """

    email = RFC822MailAddress(
        title = _( u'label-principal.IMailPassword.email',
                   u"Email address" ),
        description = _( u'help-principal.IMailPassword.email',
                         u"" ),
        required = False )

    def resetPassword():
        """Sends an email to the participant with a link to renew the password.
        The account should also be inactive. 
        """


class IInternalUser(IContained):
    """An internal user.
    """
    containers('zw.vogon.principal.interfaces.IUserFolder')


class IUserFolder(IContainer):
    """Internal user folder.
    """
    contains('zw.vogon.principal.interfaces.IInternalUser')
    
    


# to be removed
class IPerson(Interface):
    """This is the schema for an arbitrary person.
    """
    
    fullname = schema.TextLine(
        title = _( u'label-principal.IPerson.fullname',
                   u"Full name" ),
        description = _( u'help-principal.IPerson.fullname',
                         u"" ),
        required = True )

    email = RFC822MailAddress(
        title = _( u'label-principal.IPerson.email',
                   u"Email address" ),
        description = _( u'help-principal.IPerson.email',
                         u"" ),
        required = False
        )


# to be removed
class IPrincipalManagement(Interface):
    """Configlet for the principal management.
    """

    def getPrincipals():
        """Looks up the principals.
        """

class INewPortalPAUEvent(IObjectCreatedEvent):
    """A new PAU of the portal has been created.
    """

class IURLLoginCredentials(Interface):
    """Login via an URL to reset the password.
    """

