#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import email.Charset
email.Charset.add_charset('utf-8', email.Charset.SHORTEST, None, None)
import base64
from email.MIMEText import MIMEText
from datetime import datetime
import pytz

from zope.component import adapter, getUtility, getUtilitiesFor
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.sendmail.interfaces import IMailDelivery
from zope.app.authentication.principalfolder import InternalPrincipal

from zw.vogon.i18n import umlaut_normalizer
from zw.vogon.mail.settings import IMailSettings
from zw.vogon.portal.settings import IPortalSettings
from zw.vogon.principal.interfaces import IInternalUser, IPrincipalStatus, \
    IMailPassword, IUserFolder
from zw.vogon.utils.b64 import urlsafe_b64decode
from zw.vogon.utils.pwgen import generatePassword

def mail_body(mapping):
    # Translation doesn't seem to work, so do it by hand
    body = u"""The password of your user account has been reset.
Provide a new one under the following URL.

${url}

Yours sincerely,
The evaluation portal of ${portal}
"""
    for key in mapping:
        body = body.replace( '${%s}' % key, mapping[key] )
    return body


class InternalUser(InternalPrincipal):
    implements(IInternalUser, IPrincipalStatus, IMailPassword)

    active = FieldProperty(IPrincipalStatus['active'])
    remark = FieldProperty(IPrincipalStatus['remark'])
    email = FieldProperty(IMailPassword['email'])

    def checkPassword(self, password):
        return super(InternalUser, self).checkPassword(password) and self.active

    def resetPassword(self):
        # 1st generate an arbitrary password
        password = generatePassword()
        
        # 2nd set it to the principal
        self.setPassword(password)

        # 3rd activate the principal...
        self.active = True
        self.remark = u"(Re)setting password"

        # 4th send an email
        mailer = getUtility(IMailDelivery, 'zw.vogon')
        pid = base64.urlsafe_b64encode( "%s_%s" % ( self.login, password ) )

        portal_settings = getUtility(IPortalSettings)
        email_from_address = getUtility(IMailSettings).email_from_address

        mapping = { 
            'url': "%s/password.html?id=%s" % ( portal_settings.base_url,
                                                pid ),
            'portal': portal_settings.title }
        

        message = MIMEText( mail_body(mapping).encode('utf-8'),
                            'plain', 'utf-8'  )
        message['Subject'] = _( 'Password reset' )
        message['Date'] = datetime.now(pytz.utc).strftime( '%a, %d %b %Y %H:%M:%S %z')
        message['From'] = "%s <%s>" % ( umlaut_normalizer( portal_settings.title ),
                                        email_from_address )
        message['To'] = self.email
        
        # To let us find failures
        message['X-Vogon-PID'] = pid

        mailer.send( email_from_address, ( self.email, ), message.as_string() )
        


def disablePrincipal(portal, pid, reason=None):
    try:
        login, password = urlsafe_b64decode(str(pid)).split('_')
    except:
        return reason
    
    for name, user_folder in getUtilitiesFor(IUserFolder, context=portal):
        lprefix = len(user_folder.prefix)
        key = user_folder.getIdByLogin(login)
        if key is not None:
            principal = user_folder[key[lprefix:]]
            principal.active = False
            principal.remark = u"E-Mail cannot be delivered."
            return

    return reason
