#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.interface import Attribute, Interface
from zope.location.interfaces import ILocation
from z3c.form import button
from z3c.form.interfaces import IForm, IButton

class IItemForm(IForm, ILocation):
    """Base class for the form of an abstract item on a sheet,
    e.g. question, group, remark, etc.
    """

    skip_indexing = schema.Bool(
        title = _( u'label-question.IItemForm.skip_indexing',
                   u"Skip indexing this item" ),
        default = False,
        required = True )

    indices = schema.List(
        title = _( u'label-question.IItemForm.indices',
                   u"Indices" ),
        description = _( u'help-question.IItemForm.indices',
                         u"List of indices consisting of all levels "
                         u"starting with the sheet number." ),
        required = True,
        value_type = schema.Int(
            title = _( u'label-question.IItemForm.indices.value_type',
                       u"Index" ),
            ),
        )

    def index():
        """Compiles the index from the values in self.indices.
        """


class IItemGroupForm(IItemForm):
    
    subforms = Attribute(u"""List of the forms of contained items.""")

    def getContents():
        """Return contained items.
        """


class IManageItemForm(Interface):
    """
    """

    sessionKey = schema.TextLine(
        title = _( u'label-question.IManageItemForm.sessionKey',
                   u"Session key" ),
        )

class IAddItemForm(IManageItemForm):
    """
    """

    factory_name = schema.TextLine(
        title = _( u'label-question.IAddItemForm.factory_name',
                   u"Factory name" ),
        required = True )


class IManageItemGroupForm(Interface):
    
    addableTypes = Attribute(u"""List of addable types.""")

    def itemsUp(names):
        """Reorder container by moving the items 'names' up.
        """

    def itemsDown(names):
        """Reorder container by moving the items 'names' down.
        """

AddTypeVocabulary = schema.vocabulary.SimpleVocabulary( (
        schema.vocabulary.SimpleTerm(
            'text', title = u"Text question" ),
        ) )
            


class IAddableItems(Interface):
    
    addType = schema.Choice(
        title = _( u"Add" ),
        vocabulary = 'zw.vogon.adding' )

class IManageItemButtons(Interface):

    edit = button.ImageButton(
        title = _( u'Edit' ),
        image = u'zw.contentsform/edit.png',
        condition = lambda form: not ( form.editing or form.delete ) )

    up = button.ImageButton(
        title = _( u'Up' ),
        image = u'zw.contentsform/up.png',
        condition = lambda form: not (form.editing or form.delete) )
    down = button.ImageButton(
        title = _( u'Down' ),
        image = u'zw.contentsform/down.png',
        condition = lambda form: not (form.editing or form.delete) )

    delete_item = button.ImageButton(
        title = _( u'Delete' ),
        image = u'zw.contentsform/delete.png',
        condition = lambda form: not form.delete and not form.editing )

    confirm_delete = button.ImageButton(
        title = _( u'Confirm delete' ),
        image = u'zw.contentsform/confirm_delete.png',
        condition = lambda form: form.delete )
    
    cancel_delete = button.ImageButton(
        title = _( u'Cancel' ),
        image = u'zw.contentsform/cancel.png',
        condition = lambda form: form.delete )

    save = button.Button( 
        title = _( u'Save' ),
        condition = lambda form: form.editing )

    cancel_edit = button.Button(
        title = _( u'Cancel' ),
        condition = lambda form: form.editing )


class ISaveButton(IButton):
    """A button that causes the sheet data to be saved."""



class IQuestionForm(IItemForm):
    """A question subform is responsible for showing (and resp. responding)
    a question.
    """

    
