#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import adapts, getMultiAdapter
from zope.interface import implements, Interface
from zope.security.proxy import removeSecurityProxy
from zope.app.interface import queryType

from z3c.form import field, form
from z3c.form.interfaces import INPUT_MODE

from zw.vogon.question.interfaces import IQuestion, IResponse, IQuestionType
from zw.vogon.question.browser.interfaces import IQuestionForm, \
    IItemGroupForm
from zw.vogon.question.browser.form import ItemForm, ManageItemForm, \
    AddItemForm


class QuestionForm(ItemForm):
    implements(IQuestionForm)

    mode = INPUT_MODE

    @property
    def hints(self):
        return getattr(self.context, 'description', u'')

    @Lazy
    def response(self):
        return getMultiAdapter(
            ( self.context, self.request.principal ), IResponse )
    
    def getContent(self):
        return self.response

    @Lazy
    def fields(self):
        return field.Fields( removeSecurityProxy(self.context.response_schema) )



class AddQuestionForm(AddItemForm):

    fields = field.Fields(IQuestion).omit(
        '__name__', '__parent__' )


class ManageQuestionForm(QuestionForm, ManageItemForm):

   
    def update(self):
        self.ignoreContext = not self.editing
        ManageItemForm.update(self)

    def getContent(self):
        if self.editing:
            return self.context
        return super(ManageQuestionForm, self).getContent()

    @Lazy 
    def fields(self):
        if self.editing:
            question_schema = queryType(self.context, IQuestionType)
            if question_schema is None:
                return field.Fields(IQuestion).omit('__name__', '__parent__')
            else:
                return field.Fields(IQuestion, question_schema).omit(
                    '__name__', '__parent__')
        return super(ManageQuestionForm, self).fields
    
