#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _


from zope.component import adapter
from zope.interface import implementer, implementsOnly
from zope.i18n import translate
import zope.schema
import zope.schema.interfaces


from z3c.form.browser import widget, select, radio
from z3c.form.interfaces import IFormLayer, IFieldWidget
from z3c.form.widget import FieldWidget, SequenceWidget

from zw.vogon.question.choice.field import IChoiceQuestionField, \
    IMultipleChoiceQuestionField
from zw.vogon.question.choice.browser.widgets.interfaces import \
    ISelectQuestionWidget, IRadioQuestionWidget, \
    ICheckboxQuestionWidget


class SelectQuestionWidget(widget.HTMLSelectWidget, SequenceWidget):
    """
    """
    implementsOnly(ISelectQuestionWidget)

    klass = u'question-select-widget'
    items = ()
    prompt = True

    noValueMessage = None
    promptMessage = _( 'Select a value ...' )

    # Internal attributes
    _adapterValueAttributes = SequenceWidget._adapterValueAttributes + \
        ('noValueMessage', 'promptMessage')
    

    def isSelected(self, term):
        return term.token in self.value
    

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(SelectQuestionWidget, self).update()
        widget.addFieldClass(self)
        self.items = []
        if (not self.required or self.prompt) and self.multiple is None:
            self.items.append({
                    'id': self.id + '-prompt',
                    'value': self.noValueToken,
                    'content': self.promptMessage,
                    'selected': self.value == []
                    })
        for count, term in enumerate(self.terms):
            selected = self.isSelected(term)
            id = '%s-%i' % (self.id, count)
            content = term.token
            if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
                content = translate(
                    term.title, context=self.request, default=term.title)
            self.items.append(
                {'id':id, 'value':term.token, 'content':content,
                 'selected':selected})


class RadioQuestionWidget(widget.HTMLInputWidget, SequenceWidget):
    """
    """
    implementsOnly(IRadioQuestionWidget)

    klass = u'question-radio-widget'
    items = ()
    inline = False

    def isChecked(self, term):
        return term.token in self.value

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(RadioQuestionWidget, self).update()
        widget.addFieldClass(self)
        self.items = []
        for count, term in enumerate(self.terms):
            checked = self.isChecked(term)
            id = '%s-%i' % (self.id, count)
            label = term.token
            if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
                label = translate(term.title, context=self.request,
                                  default=term.title)
            self.items.append(
                {'id':id, 'name':self.name + ':list', 'value':term.token,
                 'label':label, 'checked':checked})


@adapter(IChoiceQuestionField, IFormLayer)
@implementer(IFieldWidget)
def ChoiceQuestionFieldWidget(field, request):
    """IFieldWidget factory for ChoiceQuestionField.
    """
    if field.widget_style == 'single':
        widget = FieldWidget(field, SelectQuestionWidget(request))
        if field.controlling:
            widget.onchange = u"this.form.submit();"
    else:
        widget = FieldWidget(field, RadioQuestionWidget(request))
        if field.controlling:
            widget.onclick = u"this.form.submit();"
    return widget


class CheckboxQuestionWidget(widget.HTMLInputWidget, SequenceWidget):
    """
    """
    implementsOnly(ICheckboxQuestionWidget)

    klass = u'question-checkbox-widget'
    items = ()
    inline = False

    def isChecked(self, term):
        return term.token in self.value


    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(CheckboxQuestionWidget, self).update()
        widget.addFieldClass(self)
        self.items = []
        for count, term in enumerate(self.terms):
            checked = self.isChecked(term)
            id = '%s-%i' % (self.id, count)
            label = term.token
            if zope.schema.interfaces.ITitledTokenizedTerm.providedBy(term):
                label = translate(term.title, context=self.request,
                                  default=term.title)
            self.items.append(
                {'id':id, 'name':self.name + ':list', 'value':term.token,
                 'label':label, 'checked':checked})



@adapter(IMultipleChoiceQuestionField, IFormLayer)
@implementer(IFieldWidget)
def MultipleChoiceQuestionFieldWidget(field, request):
    """IFieldWidget factory for MultipleChoiceQuestionField.
    """
    if field.widget_style == 'multiselect':
        widget = FieldWidget(field, SelectQuestionWidget(request))
        widget.multiple = 'multiple'
        widget.size = 5
    else:
        widget = FieldWidget(field, CheckboxQuestionWidget(request))
    return widget


