#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from rwproperty import getproperty, setproperty

from zope.component import adapts
from zope.interface import implements, alsoProvides, noLongerProvides
from zope.schema import TextLine
from zope.schema.fieldproperty import FieldProperty
from zope.app.container.ordered import OrderedContainer
from zope.app.interface import PersistentInterfaceClass

from zw.vogon.question.question import Question
from zw.vogon.question.interfaces import IResponseFactory, IHasTemplateForm
from zw.vogon.question.response import ResponseContained, ResponseFactory
from zw.vogon.question.control.interfaces import IControllerQuestion
from zw.vogon.question.choice.interfaces import IChoiceQuestion, IChoiceResponseSchema
from zw.vogon.question.choice.field import ChoiceQuestionField, \
    MultipleChoiceQuestionField


class ChoiceQuestion(Question):
    implements(IChoiceQuestion)

    style = FieldProperty(IChoiceQuestion['style'])
    options = FieldProperty(IChoiceQuestion['options'])
    skip_option = FieldProperty(IChoiceQuestion['skip_option'])
    other_option = FieldProperty(IChoiceQuestion['other_option'])

    @getproperty
    def enter_options_on_templating(self):
        return IHasTemplateForm.providedBy(self)
    @setproperty
    def enter_options_on_templating(self, value):
        if value:
            alsoProvides(self, IHasTemplateForm)
        else:
            noLongerProvides(self, IHasTemplateForm)
    
def updateResponseSchema(question, event):
    """Handler to keep the response_schema up to date.
    """
    if question.style in ('single', 'radio', ):
        response_field = ChoiceQuestionField(
            title = u'',
            required = question.required,
            widget_style = question.style,
            options = question.options,
            skip_option = question.skip_option,
            other_option = question.other_option,
            controlling = getattr(IControllerQuestion(question, None),
                                 'controlling', False)
            )
    else:
        response_field = MultipleChoiceQuestionField(
            title = u'',
            required = question.required,
            widget_style = question.style,
            options = question.options,
            skip_option = question.skip_option,
            other_option = question.other_option,
            )
    
    question.response_schema = PersistentInterfaceClass(
        name = 'IChoiceResponseSchema',
        bases = (IChoiceResponseSchema, ),
        attrs = { 'response': response_field,
                  'other': TextLine( title=question.other_option,
                                     required=False ) },
        __doc__ = """The concrete response schema.""",
        __module__ = 'zw.vogon.question.choice.interfaces' )



class ChoiceResponseContained(ResponseContained):

    response = None # FieldProperty(IChoiceResponseSchema['response'])
    other = None

class ChoiceResponseFactory(ResponseFactory):
    implements(IResponseFactory)
    adapts(IChoiceQuestion)

    title = _( u'Choice response factory' )
    description = _( u'This factory instantiates new choice responses.' )
    _callable = ChoiceResponseContained
