#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zope import schema
from zope.component import queryUtility
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
from zope.interface import implements

from plone.i18n.normalizer.interfaces import IIDNormalizer

from zw.vogon.question.field import QuestionField, IQuestionField


class IChoiceQuestionField(IQuestionField):
    """Field to store the answer to a choice question.
    """

class IMultipleChoiceQuestionField(IQuestionField):
    """Field to store the answers to a multiple choice question.
    """


class dummyNormalizer(object):

    def normalize(self, value):
        return str(value)

class ChoiceQuestionField(schema.Choice, QuestionField):
    implements(IChoiceQuestionField)
    
    controlling = False

    def __init__(self, options=None, skip_option=None,
                 other_option=None, widget_style=None, controlling=False, **kw):
        
        self.options = options
        self.skip_option = skip_option
        self.other_option = other_option
        self.widget_style = widget_style
        self.controlling = controlling

        values = []
        if skip_option is not None:
             values.append(skip_option)
        if options is not None:
            values += options
        if other_option is not None:
            values.append(other_option)

        normalizer = queryUtility(IIDNormalizer, default=dummyNormalizer)

        vocabulary = SimpleVocabulary( tuple(
                [ SimpleTerm( value, token = "%s-%i" % ( normalizer.normalize(value), i ), title=value ) \
                      for i, value in enumerate(values) ] ) )

        super(ChoiceQuestionField, self).__init__(vocabulary = vocabulary, **kw)


class MultipleChoiceQuestionField(schema.Set, QuestionField):
    implements(IMultipleChoiceQuestionField)


    def __init__(self, options=None, skip_option=None,
                 other_option=None, widget_style=None, **kw):
        
        self.options = options
        self.skip_option = skip_option
        self.other_option = other_option
        self.widget_style = widget_style

        values = []
        # if skip_option is not None:
        #     values.append(skip_option)
        if options is not None:
            values += options
        if other_option is not None:
            values.append(other_option)

        normalizer = queryUtility(IIDNormalizer, default=dummyNormalizer)

        vocabulary = SimpleVocabulary( tuple(
                [ SimpleTerm( value, token = "%s-%i" % ( normalizer.normalize(value), i ), title=value ) \
                      for i, value in enumerate(values) ] ) )

            
        super(MultipleChoiceQuestionField, self).__init__(
            value_type = schema.Choice(vocabulary = vocabulary), **kw)

