#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.interface import Interface

from zw.vogon.question.interfaces import IResponsable, \
    IResponseSchema, IQuestionField
from zw.vogon.question.choice.field import ChoiceQuestionField


ChoiceStyleVocabulary = schema.vocabulary.SimpleVocabulary( (
        schema.vocabulary.SimpleTerm( # widgets.SelectQuestionWidget,
                                      'single', title = _(
                u'label-question.choice.ChoiceStyleVocabulary.single',
                u"Single selection" )  ),
        schema.vocabulary.SimpleTerm( # widgets.RadioQuestionWidget,
                                      'radio', title = _(
                u'label-question.choice.ChoiceStyleVocabulary.radio',
                u"Radio buttons" ) ),
        schema.vocabulary.SimpleTerm( #widgets.MultiSelectQuestionWidget,
                                      'multiselect', title = _(
                u'label-question.choice.ChoiceStyleVocabulary.multiselect',
                u"Multiple selection" ) ),
        schema.vocabulary.SimpleTerm( # widgets.CheckboxQuestionWidget,
                                      'checkbox', title = _(
                u'label-question.choice.ChoiceStyleVocabulary.checkboxes',
                u"Checkboxes" ) )
        ) )


class IChoiceQuestion(Interface):
    """A choice question.
    """

    style = schema.Choice(
        title = _( u'label-question.choice.IChoice.style',
                   u"Style" ),
        description = _( u'help-question.choice.IChoice.style',
                         u"Choose the selection widget." ),
        required = True,
        vocabulary = ChoiceStyleVocabulary )

    options = schema.List(
        title = _( u'label-question.choice.IChoice.options',
                   u"Answer options" ),
        description = _( u'help-question.choice.IChoice.options',
                         u"" ),
        required = False,
        unique = True,
        value_type = schema.TextLine(
            title = _( u'label-question.choice.IChoice.options.value_type',
                       u"Answer" ),
            description = _( u'help-question.choice.IChoice.options.value_type',
                             u"" ),
            )
        )

    skip_option = schema.TextLine(
        title = _( u'label-question.choice.IChoice.skip_option',
                   u"'No response' option" ),
        description = _( u'help-question.choice.IChoice.skip_option',
                         u"Enter an answer option for skipping this question. Note that it will be ignored for required questions, but it is mandatory in the other case." ),
        required = False )

    other_option = schema.TextLine(
        title = _( u'label-question.choice.IChoice.other_option',
                   u"'Other' option" ),
        description = _( u'help-question.choice.IChoice.other_option',
                         u"Choose an option label to let the participant specify another option in an extra text field or leave blank." ),
        required = False,
        default = u'' )

    enter_options_on_templating = schema.Bool(
        title = _(
            u'label-question.choice.IChoice.enter_options_on_templating',
            u"Specify options on creating the survey from template." ),
        description = _(
            u'help-question.choice.IChoice.enter_options_on_templating',
            u"If You check this box, you will be able to modify the options on survey creation from template." ),
        required = True,
        default = False
        )

    #@invariant
    #def requiredAndSkipOption(question):
    #    if not ( question.required or question.skip_option ):
    #        raise Invalid( _(
    #                u'error-question.choice.IChoice.requiredAndSkipOption',
    #                u"You need tos specify a 'No response' option, if answering the question is not mandatory." ) )


class IChoiceResponseSchema(IResponseSchema):
    """
    """

    response = ChoiceQuestionField( title = u'Response' )
    other = schema.TextLine( title = u'Other', required = False )
