#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from StringIO import StringIO

from zope.cachedescriptors.property import Lazy
from zope.component import adapts
from zope.interface import implements
from z3c.pagelet.browser import BrowserPagelet

from zw.vogon.question.results.latex import cleanSRC
from zw.vogon.question.results.results import QuestionResults
from zw.vogon.question.results.renderer import HTMLResultRenderer, \
    LaTeXResultRenderer, XLSResultRenderer

from zw.vogon.question.choice.interfaces import IChoiceQuestion
from zw.vogon.question.choice.results.interfaces import IChoiceQuestionResults


class ChoiceQuestionResults(QuestionResults):
    implements(IChoiceQuestionResults)
    adapts(IChoiceQuestion)

    @Lazy
    def baseOptionsMapping(self):
        sum = self.count - self.numberOfSkipped
        if sum < 1:
            return tuple()
        options = list(self.context.options)
        if self.context.skip_option:
            options.append(self.context.skip_option)
        if self.context.other_option:
            options.append(self.context.other_option)
        mapping = {}.fromkeys(options, 0)
        
        for response in self._core_responses:
            # We collect only predefined options.
            responses = response.response
            if type(responses) is not set:
                responses = set([responses])
            for response in responses:
                if response in mapping:
                    mapping[response] += 1
            
        return tuple( [ { 'label': option,
                          'absolute': mapping[option],
                          'relative': float(mapping[option])/sum,
                          } for option in options ] )


    @Lazy
    def extraOptionsMapping(self):
        sum = self.count - self.numberOfSkipped
        if sum < 1:
            return tuple()
        mapping = {}
        base_options = self.context.options

        for response in self._core_responses:
            # We collect only extra options.
            option = response.response
            if option in base_options:
                if option in mapping:
                    mapping[option] = mapping.get(option, 0) + 1

            
        return tuple( [ { 'label': option,
                          'absolute': mapping[option],
                          'relative': float(mapping[option])/sum,
                          } for option in mapping.keys() ] )

    @Lazy
    def optionsMapping(self):
        options = []
        for option in self.baseOptionsMapping:
            noption = option.copy()
            noption['base'] = True 
            options.append(noption)
        for option in self.extraOptionsMapping:
            noption = option.copy()
            noption['base'] = False 
            options.append(noption)

        return tuple(options)

    
class ChoiceQuestionResultsPage(HTMLResultRenderer, BrowserPagelet, 
                                ChoiceQuestionResults):


    def __init__(self, context, request):
        super(ChoiceQuestionResultsPage, self).__init__(context, request)
        ChoiceQuestionResults.__init__(self, context)


choicetex = u"""%%%% question: tabular
\\noindent\\parbox[t]{.48\\textwidth}{
  \\noindent\\textbf{%(index)s\\hspace{2mm} %(title)s} ($%(count)i\\times$)\\\\[1mm]
  \\begin{tabularx}{.46\\textwidth}{X|>{\\centering}p{3ex}|>{\\centering}p{3ex}}
  %(values)s
  \\end{tabularx}
}
\\medskip"""


class ChoiceQuestionResultsTeX(LaTeXResultRenderer, ChoiceQuestionResults):

    def __init__(self, context, request):
        super(ChoiceQuestionResultsTeX, self).__init__(context, request)
        ChoiceQuestionResults.__init__(self, context)

    def render(self):
        self.update()

        texsrc = StringIO()
                
        print >> texsrc, choicetex % {
            'index': self.index,
            'title': cleanSRC(self.context.title),
            'count': self.count,
            'values': u'\n'.join( 
                [ u"%s & %i & %.1f\\%% \\tabularnewline" \
                      % (cleanSRC(item['label']), #
                         item['absolute'], item['relative']*100)\
                      for item in self.baseOptionsMapping + self.extraOptionsMapping ] ),
            }
        
        return texsrc.getvalue()

      
class ChoiceQuestionResultsXLS(XLSResultRenderer, ChoiceQuestionResults):
    
    ws = None
    col = 0
    response_mapping = {}

    def __init__(self, context, request):
        super(ChoiceQuestionResultsXLS, self).__init__(context, request)
        ChoiceQuestionResults.__init__(self, context)
        
    def update(self):
        self.ws.write( 0, self.col, self.index ) 
        self.ws.write( 1, self.col, self.context.title )
        for r in self._core_responses:
            if r.participant not in self.response_mapping:
                self.response_mapping[r.participant] = len(self.response_mapping)+1
            response = ''
            if type(r.response) is set:
                response = u', '.join(r.response)
            elif r.response is not None:
                response = r.response
                
            self.ws.write( self.response_mapping[r.participant]+2, self.col,
                           response )

        self.col += 1
  
