#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _

from persistent import Persistent

from zope.component import adapts, adapter, getMultiAdapter, getUtility
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.api import getParent, getName
from zope.app.container.contained import Contained
from zope.app.generations.utility import findObjectsProviding
from zope.app.intid.interfaces import IIntIds

from z3c.objpath import path, resolve

from zw.vogon.survey.interfaces import ISheetContainer
from zw.vogon.template.interfaces import IPostCloningEvent, IClone
from zw.vogon.utils.mapping import AnnotationsMapper, MappingFieldProperty

from zw.vogon.question.interfaces import IResponse
from zw.vogon.question.control.interfaces import IControlledQuestion, \
    IControllerQuestion, ICanControl, IControllable, IHideAction


class HideAction(Persistent, Contained):
    implements(IHideAction)
    
    option = FieldProperty(IHideAction['option'])
    slave = FieldProperty(IHideAction['slave'])

    def __init__(self, option, slave):
        self.option = option
        self.slave = slave
    
    def __eq__(self, other):
        if isinstance(other, HideAction):
            return self.option == other.option and self.slave == other.slave
        return False

class ControlledQuestion(AnnotationsMapper):
    adapts(IControllable)
    implements(IControlledQuestion)

    KEY = 'zw.vogon.question.controlled'

    controllers = MappingFieldProperty(IControlledQuestion['controllers'])

    def isHidden(self, principal):
        hidden = False
        for controller in self.controllers:
            # Get the value of the principal
            try:
                response = getMultiAdapter(
                    ( controller, principal ), IResponse)
            except:
                continue

            else:
                for action in IControllerQuestion(controller).skip_items:
                    if action == HideAction(response.response, getParent(self)):
                        hidden = True
        return hidden


class ControllerQuestion(AnnotationsMapper):
    adapts(ICanControl)
    implements(IControllerQuestion)

    KEY = 'zw.vogon.question.controller'

    skip_items = MappingFieldProperty(IControllerQuestion['skip_items'])

    def hideObjectOnOptions(self, obj, options):
        for option in options:
            action = HideAction(option, obj)
            self.skip_items.update([action])

    def removeActionsForObject(self, obj):
        for action in self.skip_items:
            if action.slave == obj:
                self.skip_items.remove(action)

    @property
    def controlling(self):
        return False
    



@adapter(ISheetContainer, IPostCloningEvent)
def updateControlling(survey, event):
    survey_template = event.template

    for question in findObjectsProviding(survey, IControllable):
        template = IClone(question).template
        cqs = []
        for controller_template in IControlledQuestion(template).controllers:
            qpath = path(survey_template, controller_template)[len(getName(survey_template))+1:]
            cq = resolve(survey, '/'+getName(survey)+qpath)
            cqs.append(cq)
        IControlledQuestion(question).controllers = set(cqs)
            

    for question in findObjectsProviding(survey, ICanControl):
        template = IClone(question).template
        
        actions = []
        for action in IControllerQuestion(template).skip_items or []:
            spath = path(survey_template, action.slave)[len(getName(survey_template))+1:]
            slave = resolve(survey, '/'+getName(survey)+spath)
            actions.append(HideAction(action.option, slave))
        IControllerQuestion(question).skip_items = set(actions)
        
        
