#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.annotation.interfaces import IAnnotatable
from zope.interface import Attribute, Interface

from zw.schema.reference import Reference

class IControllable(IAnnotatable):
    """Mark a sheet item controllable.
    """

class ICanControl(IAnnotatable):
    """Mark a sheet item controller.
    """


class IControlledQuestion(Interface):

    controllers = schema.Set(
        title = _( u"Controllers" ),
        description = _( u"" ),
        default = set(),
        value_type = Reference(
            title = u"Controller",
            schemata = ( ICanControl, )
            )
        )
    
    def isHidden(principal):
        """Check
        """

class IHideAction(Interface):
    """Hide question on option
    """

    option = schema.TextLine( title = u"Option", required=False )
    slave = Reference( title = u"Slave" )

class IControllerQuestion(Interface):

    skip_items = schema.Set(
        title = u"Skip items on options",
        default = set(),
        value_type = schema.Object(
            title = u"Controlled question",
            schema = IHideAction,
            )
        )

    controlling = schema.Bool(
        title = u"Is the question actually controlling",
        default = False )



class IControlAction(Interface):
    """
    """

    permit_other_actions = schema.Bool(
        title = u"Permit other control actions",
        default = True )

    
    def apply(view):
        """Modifies the view.
        """

class IController(IAnnotatable):
    """Marker class that allows a control annotations to questions.
    """

    actions = schema.Set(
        title = u"Control actions",
        description = u"List of control actions",
        value_type = schema.Object(
            title = u"Action",
            schema = IControlAction ),
        )


class IControlRegistry(Interface):
    """A utility to apply controlling on sheet items.
    """

    
