#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.annotation.interfaces import IAnnotatable
from zope.component.interfaces import IFactory
from zope.interface import Attribute, Interface
from zope.interface.interfaces import IInterface
from zope.schema.interfaces import IField
from zope.app.container.constraints import containers, contains
from zope.app.container.interfaces import IContained, IContainer, \
    IOrderedContainer
from zope.app.file.interfaces import IImage


class ISheetItem(IContained):
    """A contained question (or another item on a sheet).
    """
    containers('zw.vogon.question.interfaces.IQuestionContainer')


class IQuestionContainer(IOrderedContainer):
    """Stores questions or other IGroupItems-
    """
    contains('zw.vogon.question.interfaces.ISheetItem',)



##############################################################################

class IQuestion(Interface):
    """An abstract base question.
    """

    title = schema.TextLine(
        title = _( u'label-question.IQuestion.title',
                   u"Title" ),
        description = _( u'help-question.IQuestion.title',
                         u"" ),
        required = True )

    description = schema.Text(
        title = _( u'label-question.IQuestion.description',
                   u"Explanation" ),
        description = _( u'help-question.IQuestion.description',
                         u"Some details to the question." ),
        required = False )

    required = schema.Bool(
        title = _( u'label-question.IQuestion.required',
                   u"Required" ),
        description = _( u'help-question.IQuestion.required',
                         u"Check this box whether answering the question is mandatory." ),
        default = False,
        required = True )

    #question_type = schema.Choice(
    #    title = _( u'label-question.IQuestion.question_type',
    #               u"Type of question" ),
    #    description = _( u'help-question.IQuestion.question_type',
    #                     u"Select the kind of this question." ),
    #    required = True,
    #    vocabulary = u'Vogon Question Types' )


class IQuestionImage(Interface):
    """Extends a question by showing an image.
    """
    
    image = schema.Object(
        title = _( u'label-question.IQuestionImage.image',
                   u"Image" ),
        description = _( u'help-question.IQuestionImage.image',
                         u"Upload an optional image for the question." ),
        required = False,
        default = None,
        schema = IImage )

##############################################################################

class IQuestionType(IInterface):
    """This interface is implemented by all concrete question types.
    """


##############################################################################


class IResponseSchema(Interface):
    """Abstract base interface for a schema describing
    the response fields corresponding to the question.
    """

class IResponsable(Interface):
    """A responsable question should provide a suitable response schema.
    """

    response_schema = schema.InterfaceField(
        title = _( u'label-question.IResponsable.__schema__',
                   u"The response schema." ),
        description = _( u'help-question.IResponsable.__schema__',
                         u"" ),
        default = IResponseSchema,
        required = True )

class IStoreResponses(IAnnotatable):
    """Annotation to the object the responses store.
    """

#############################################################################


class IResponse(Interface):
    """A response adapts to a (IResponsable) question
    and directly provide it's response schema.
    """

    __question__ = schema.Object(
        title = _( u'label.question.IResponse.__question__',
                   u"The corresponding question." ),
        schema = IQuestion )
        

class IResponseContainer(IContainer):
    """The Response container.
    Collects persistently the responses of a question.
    """
    contains('zw.vogon.question.interfaces.IResponseContained')


class IResponseFolder(IContained):
    """Response folder.
    """
    containers('zw.vogon.question.interfaces.IResponseable')

    def getResponseFor(principal):
        """Looks up the reponse for principal
        and if necessary creates a new one.
        """

    def createBlankResponseFor(participant, force=False):
        """Create or blank a response for participant.
        """

class IResponseContained(IContained):
    """This is the persistent part of a IResponse assigned to a participant.
    """
    containers('zw.vogon.question.interfaces.IResponseContainer')

    participant = schema.ASCIILine(
        title = _( u'label-question.IResponseContained.participant',
                   u"Participant's principal id" ),
        description = _( u'help-question.IResponseContained.participant',
                         u"Stores the participant's unique principal id." ),
        required = True )

    readonly = schema.Bool(
        title = _( u'label-question.IResponseContained.readonly',
                   u"Readonly response object" ),
        description = _( u'help-question.IResponseContained.readonly',
                         u"Lock question after finishing survey" ),
        default = True,
        required = True )

class IResponseFactory(IFactory):
    """This factory adapts to a question to create a contained response.
    """

    __question__ = schema.Object(
        title = _( u'label.question.IResponseFactory.__question__',
                   u"The based question." ),
        schema = IResponsable )


##############################################################################
# Base class for question fields

class IQuestionField(IField):
    """Abstract question field.
    """

##############################################################################

class IHasTemplateForm(Interface):
    """Marker: Change question on templating.
    """
