#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import getMultiAdapter
from zope.event import notify
from zope.lifecycleevent import ObjectModifiedEvent
from zope.location import locate
from zope.pagetemplate.interfaces import IPageTemplate
from zope.publisher.browser import BrowserPage
from zope.traversing.api import getParent
from zope.viewlet.viewlet import CSSViewlet
from z3c.form import button, field, form, subform
from z3c.form.interfaces import INPUT_MODE

from zw.vogon.question.browser.form import ManageItemForm
from zw.vogon.question.rating.interfaces import IRatingOption, IRatingQuestion
from zw.vogon.question.rating.option import RatingOption, \
    RatingOptionList


MAX_OPTIONS_ROWS = 3

RatingOptionCSSViewlet = CSSViewlet('rating_option.css')

class RatingOptionForm(subform.EditSubForm):
    form.extends(subform.EditSubForm)
    
    fields = field.Fields(IRatingOption)
    index = None
    error = None
    mode = INPUT_MODE

    noChangesMessage = None

    def updateWidgets(self):
        super(RatingOptionForm, self).updateWidgets()
        for name, widget in self.widgets.items():
            widget.addClass(u'rating-option-%s' % name)

    @button.handler(form.AddForm.buttons['add'])
    def handleAdd(self, action):
        self.handleApply(self, action)

    @button.handler(ManageItemForm.buttons['save'])
    def handleSave(self, action):
        self.handleApply(self, action)

    def handleApply(self, subform, action):
        data, errors = self.widgets.extract()
        if errors:
            self.status = self.formErrorsMessage
            return
        content = self.getContent()
        changed = form.applyChanges(self, content, data)
        if changed:
            notify(ObjectModifiedEvent(content))
            notify(ObjectModifiedEvent(getParent(content)))
            self.status = self.successMessage
        else:
            self.status = self.noChangesMessage



class RatingOptionsForm(BrowserPage):
    
    forms = ()
    prefix = 'rating-options.'
    label = _( u"Rating options" )
    parentForm = None
    status = None

    changed = False
    base_options = None

    def __init__(self, context, request, parentForm):
        super(RatingOptionsForm, self).__init__(context, request)
        self.__parent__ = self.parentForm = parentForm
        self.prefix = parentForm.prefix + 'rating-options.'
        self.session = parentForm.session

    def update(self):
        base_options = self.getContents()
        while len(base_options) < MAX_OPTIONS_ROWS:
            option = RatingOption()
            locate(option, self.context)
            base_options.append( option )
        base_options.sort()
        self.base_options = base_options

        forms = []
        for i, option in enumerate(base_options):
            form = RatingOptionForm(option, self.request, self.parentForm)
            form.index = i
            form.prefix = self.prefix + str(i) + '.'
            form.update()
            forms.append(form)
        self.forms = tuple(forms)
        
    def render(self):
        template = getMultiAdapter( (self, self.request), IPageTemplate )
        return template(self)

    def getContents(self):
        # Get the options from the rating question
        if IRatingQuestion.providedBy(self.context):
            return self.context.base_options
        # We are in the process of adding a question, so store the
        # option list in the session variable.
        if self.prefix not in self.session:
            self.session[self.prefix] = RatingOptionList()
        return self.session[self.prefix]


class DeprRatingOptionsForm(form.AddForm):
    fields = field.Fields(IRatingOption)
    
    parentForm = None
    

    def updateWidgets(self):
        super(RatingOptionsForm, self).updateWidgets()
        for name, widget in self.widgets.items():
            widget.addClass(u'rating-option-%s' % name)

    def __init__(self, context, request, parentForm):
        super(RatingOptionsForm, self).__init__(context, request)
        self.__parent__ = self.parentForm = parentForm
        self.session = parentForm.session
        self.prefix = str(parentForm.prefix + 'base-options.')

    def create(self, data):
        option = RatingOption(**data)
        return option

    def add(self, obj):
        self.getContent().append(obj)
        locate(obj, self.getContent())
        self.ignoreRequest = True
        self.updateWidgets()
        if IRatingQuestion.providedBy(self.context):
            notify(ObjectModifiedEvent(getParent(self.getContent())))
        return obj

    
    def update(self):
        super(RatingOptionsForm, self).update()
        self.optionForms = []    
        for index, option in enumerate(self.getContent()):
            option_form = RatingOptionForm(option, self.request,
                                           self.parentForm)
            option_form.index = index
            option_form.prefix = self.prefix + str(index) + '.'
            option_form.update()
            if not option_form.deleted:
                self.optionForms.append(option_form)
                
            if option_form.status is option_form.successMessage and \
                    IRatingQuestion.providedBy(self.context):
                notify(ObjectModifiedEvent(getParent(self.context)))
                

    def render(self):
        template = getMultiAdapter( (self, self.request), IPageTemplate )
        return template(self)
