#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import adapts, createObject
from zope.event import notify
from zope.interface import implements, Interface
from zope.lifecycleevent import ObjectModifiedEvent
from zope.location import locate

from z3c.form import button, field, form

from zw.vogon.question.interfaces import IQuestion, IQuestionContainer
from zw.vogon.question.browser.question_form import QuestionForm, \
    AddQuestionForm, ManageQuestionForm
from zw.vogon.question.browser.interfaces import IItemGroupForm, \
    IManageItemGroupForm, IManageItemButtons

from zw.vogon.question.rating.option import RatingOptionList
from zw.vogon.question.rating.browser.interfaces import IRatingQuestionForm, \
    IManageRatingQuestionForm, IAddRatingQuestionForm
from zw.vogon.question.rating.browser.rating_option import RatingOptionsForm
from zw.vogon.question.rating.interfaces import IRatingQuestion


class RatingQuestionForm(QuestionForm):
    implements(IRatingQuestionForm)
    adapts(IRatingQuestion, Interface, IItemGroupForm)


class ManageRatingQuestionForm(ManageQuestionForm):
    implements(IRatingQuestionForm, IManageRatingQuestionForm)
    adapts(IRatingQuestion, Interface, IManageItemGroupForm)
    form.extends(ManageQuestionForm, ignoreFields=True)

    optionsForm = None

    @Lazy 
    def fields(self):
        if self.editing:
            return field.Fields(IQuestion, IRatingQuestion).omit(
                '__name__', '__parent__', 'base_options' )
        return super(ManageQuestionForm, self).fields

    def update(self):
        super(ManageRatingQuestionForm, self).update()

        if self.editing:

            self.optionsForm = RatingOptionsForm(self.context, 
                                                 self.request, self)
            self.optionsForm.update()

    @button.handler( IManageItemButtons['save'] )
    def handleSave(self, action):
        self.handleApply(self, action)
        if self.optionsForm.errors:
            return

        if self.editing:
            self.session['edit'].remove(self.prefix)
        self.request.response.redirect(self.request.getURL())


class AddRatingQuestionForm(AddQuestionForm):
    implements(IAddRatingQuestionForm)
    adapts(IQuestionContainer, Interface, IManageItemGroupForm)

    factory_name = u'zw.vogon.question.RatingQuestion'
    fields = field.Fields(IQuestion, IRatingQuestion).omit(
        '__name__', '__parent__', 'options', 'base_options' )

    def update(self):
        #super(AddRatingQuestionForm, self).update()
        self.updateWidgets()
        self.updateActions()

        self.optionsForm = RatingOptionsForm(self.context, self.request, self)
        self.optionsForm.update()

        self.actions.execute()
        

    def create(self, data):
        obj = createObject( self.factory_name )
        changes = form.applyChanges(self, obj, data)
        
        obj.base_options = self.optionsForm.getContents()
        locate(obj.base_options, obj, 'base_options')
        for i, option in enumerate(obj.base_options):
            locate(option, obj, str(i))
        del self.session[self.optionsForm.prefix]

        return obj
