#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import adapts, createObject, getUtility
from zope.interface import implements, Interface
from zope.location import locate
from zope.session.interfaces import ISession
from zope.traversing.api import getParent, getName
from zope.app.container.constraints import checkObject
from zope.app.container.interfaces import INameChooser

from z3c.form import button, field
from z3c.form.interfaces import DISPLAY_MODE
from z3c.formui import form

from zw.contentsform.adding import AddingFormMixin
from zw.contentsform.column import FieldColumn, ContentNameColumn, Columns, \
    FormActionsColumn
from zw.contentsform.contentsform import ContentsForm
from zw.contentsform.cutcopypaste import CutCopyPasteMixin
from zw.contentsform.row import AddContentRow, EditContentRow

from zw.vogon.question.rating.interfaces import IRatingTemplate, \
    IRatingTemplates
from zw.vogon.question.rating.browser.rating_option import RatingOptionsForm

class ManageRatingTemplates(AddingFormMixin, CutCopyPasteMixin, ContentsForm):
    implements(IRatingTemplates)
    form.extends(AddingFormMixin, CutCopyPasteMixin)


    label = _( u'label-question-rating.browser.ManageRatingTemplates.label',
               u"Rating question templates" )

    cssClasses = {'table': 'listing' }
    canCut = False
    canCopy = False
    canPaste = False

    @Lazy
    def container(self):
        principal_folder = getUtility(IRatingTemplates)
        return principal_folder

    #def getContents(self):
    #    #pau = getUtility(IAuthentication)
    #    for item in self.container.values():
    #        yield self.container.principalInfo(
    #            self.container.getIdByLogin(item.login) )

    def _getColumns(self):
        return Columns(
            FieldColumn( IRatingTemplate['title'], title=_( u'Name' ) ),
            # Preview
            FormActionsColumn()
            )


    @button.buttonAndHandler( _( u'Add template' ), name='add_template' )
    def handleAddTemplate(self, action):
        self.request.response.redirect(self.request.getURL(1)+'/@@add.html')

class EditRatingTemplate(form.EditForm):
    label = _( u"Edit rating template" )
    fields = field.Fields(IRatingTemplate).omit(
        '__name__', '__parent__', 'base_options')

    @Lazy
    def session(self):
        session = ISession(self.request)['edit_rating_template']
        return session

    def update(self):
        super(EditRatingTemplate, self).update()

        self.optionsForm = RatingOptionsForm(self.context, self.request, self)
        self.optionsForm.update()


_marker = object()

class AddRatingTemplate(form.AddForm):
    label = _( u"Add a rating template" )
    fields = field.Fields(IRatingTemplate).omit(
        '__name__', '__parent__', 'base_options')


    @Lazy
    def session(self):
        session = ISession(self.request)['add_rating_template']
        return session


    def update(self):
        #super(AddRatingTemplate, self).update()
        self.updateWidgets()
        self.updateActions()

        self.optionsForm = RatingOptionsForm(self.context, self.request, self)
        self.optionsForm.update()

        self.actions.execute()
        
    def create(self, data):
        obj = createObject( u'zw.vogon.question.RatingTemplate' )
        changes = form.applyChanges(self, obj, data)
        
        #obj.base_options = self.optionsForm.getContents()
        #locate(obj.base_options, obj, 'base_options')
        #for i, option in enumerate(obj.base_options):
        #    locate(option, obj, str(i))

        return obj

    def add(self, content):
        name = getattr(content, '__name__', None)
        chooser = INameChooser(self.context)

        # check precondition
        checkObject(self.context, name, content)

        name = chooser.chooseName(name or '', content)
        chooser.checkName(name, content)

        self.context[name] = content
        return self.context[name]
        
    def nextURL(self):
        return self.request.getURL(1)
