#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _


from zope.component import adapter, adapts
from zope.interface import implementer, implementsOnly, implements
from zope.i18n import translate
from zope.schema.interfaces import ITitledTokenizedTerm


from z3c.form.converter import BaseDataConverter
from z3c.form.browser import widget, select, radio
from z3c.form.interfaces import IFormLayer, IFieldWidget, IDataConverter
from z3c.form.widget import FieldWidget, SequenceWidget

from zw.vogon.question.rating.field import IRatingQuestionField
from zw.vogon.question.rating.browser.widgets.interfaces import \
    IRatingQuestionWidget


class RatingQuestionWidget(widget.HTMLInputWidget, SequenceWidget):
    """
    """
    implementsOnly(IRatingQuestionWidget)

    klass = u'question-rating-widget'
    items = ()
    #prompt = False
    inline = False
    items = []

    def isChecked(self, term):
        return term.token in self.value

    def update(self):
        """See z3c.form.interfaces.IWidget."""
        super(RatingQuestionWidget, self).update()
        widget.addFieldClass(self)
        items = []
        last_regular_num = 0
        for count, term in enumerate(self.terms):
            checked = self.isChecked(term)
            id = '%s-%i' % (self.id, count)
            color = term.color
            label = term.label
            title = ( not term.extra_term ) and term.token or ''
            if not term.extra_term:
                last_regular_num = count

            items.append(
                {'id':id, 
                 'name':self.name + ':list',
                 'value':term.token,
                 'title': title, 
                 'label':label, 
                 'checked':checked, 
                 'klass': term.extra_term and 'extra-term' or 'option',
                 'color': color,
                 'rating_label_class': 'rating-label mid'})

        items[0]['rating_label_class'] = 'rating-label left'
        items[last_regular_num]['rating_label_class'] = 'rating-label right'
            
        self.items = items

        

@adapter(IRatingQuestionField, IFormLayer)
@implementer(IFieldWidget)
def RatingQuestionFieldWidget(field, request):
    """IFieldWidget factory for RatingQuestionField.
    """
    widget = FieldWidget(field, RatingQuestionWidget(request))
    return widget


class RatingQuestionConverter(BaseDataConverter):
    adapts(IRatingQuestionField, IRatingQuestionWidget)
    implements(IDataConverter)

    def toWidgetValue(self, value):
        """See z3c.form.interfaces.IDataConverter
        """
        widget = self.widget
        if value is self.field.missing_value:
            return []
        # Look up the term in the terms
        terms = widget.updateTerms()
        return [terms.getTerm(value).token]

    def toFieldValue(self, value):
        """See z3c.form.interfaces.IDataConverter
        """
        widget = self.widget
        if not len(value) or value[0] == widget.noValueToken:
            return self.field.missing_value
        terms = widget.updateTerms()
        return widget.terms.getValue(value[0])


#class RatingQuestionTerms(Terms):
#    adapts(Interface, IFormLayer, Interface, IRatingQuestionField, IWidget)
#    
#    def __init__(self, context, request, form, field, widget):
#        self.context = context
#        self.request = request
#        self.form = form
#        self.field = field
#        self.widget = widget
#        if field.vocabulary is None:
#            field = field.bind(context)
#        self.terms = field.vocabulary
