#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.schema.interfaces import ITokenizedTerm
from zope.interface import Interface, invariant
from zope.app.container.constraints import contains, containers
from zope.app.container.interfaces import IContainer, IContained

from zw.schema.color import Color
from zw.vogon.question.interfaces import IResponsable, \
    IResponseSchema, IQuestionField
from zw.vogon.question.rating.field import RatingQuestionField



class IRatingOption(Interface):
    """Store a rating option
    """

    value = schema.Int(
        title = _( u'label-question.rating.IRatingOption.value',
                   u"Value", ),
        description = _( u'help-question.rating.IRatingOption.value',
                         u"Range(-10, ..., 20)" ),
        min = -10,
        max = 20,
        required = False
        )

    label = schema.TextLine(
        title = _( u'label-question.rating.IRatingOption.label',
                   u"Label", ),
        description = _( u'help-question.rating.IRatingOption.label',
                         u"" ),
        required = False
        )

    color = Color(
        title = _( u'label-question.rating.IRatingOption.color',
                   u"Color", ),
        description = _( u'help-question.rating.IRatingOption.color',
                         u"Format: #RRGGBB" ),
        required = False
        )


class IRatingOptionTerm(ITokenizedTerm):
    """An ordinary SimpleTerm extended by label and color.
    """

    label = schema.TextLine(
        title = _( u'label-question.rating.IRatingOption.label',
                   u"Label", ),
        description = _( u'help-question.rating.IRatingOption.label',
                         u"" ),
        required = False
        )

    color = Color(
        title = _( u'label-question.rating.IRatingOption.color',
                   u"Color", ),
        description = _( u'help-question.rating.IRatingOption.color',
                         u"Format: #RRGGBB" )
        )
    
    extra_term = schema.Bool(
        title = _( u'label-question.rating.IRatingOption.extra_term',
                   u"Extra term" ),
        description = _( u'help-question.rating.IRatingOption.extra_term',
                         u"If True this term is handled apart from the regular ones. "
                         u"This is useful e.g. for the 'no_rating' term." ),
        required = True,
        default = False
        )


class IRatingQuestion(Interface):
    """A rating question.
    """

    #template

    no_ratings = schema.List(
        title = _( u'label-question.rating.IRating.no_ratings',
                   u"Alternative options" ),
        description = _( u'help-question.rating.IRating.no_ratings',
                         u"This allows the participant to skip this rating." ),
        required = False,
        default = [],
        value_type = schema.TextLine(
            title = u"No rating option" ),
            )

    base_options = schema.List(
        title = _( u'label-question.rating.IRating.base_options',
                   u"Base options", ),
        description = _( u'help-question.rating.IRating.base_options',
                         u"Enter the values of possible ratings." ),
        value_type = schema.Object(
            title = _( u'label-question.rating.IRating.base_options.value_type',
                       u"Option", ),
            description = _(
                u'help-question.rating.IRating.base_options.value_type',
                u"" ),
            schema = IRatingOption
            )
        )
    
    #@invariant
    #def checkBaseOptions(question):
    #    if False:
    #        raise Invalid( _( u'' ) )
    #    pass


    def generateOptionsVocabulary():
        """Generate a vocabulary from base_options for all options.
        """



##############################################################################

class IRatingTemplateContainer(IContainer):
    """Stores rating templates.
    """
    contains('zw.vogon.question.rating.interfaces.IRatingTemplate')



class IRatingTemplate(IContained):
    """Stores a rating template.
    """
    containers('zw.vogon.question.rating.interfaces.IRatingTemplateContainer')

    title = schema.TextLine(
        title = _( u'label-question.rating.IRatingTemplate.title',
                   u"Title" ),
        description = _( u'help-question.rating.IRatingTemplate.title',
                         u"Unique identifier for this type of rating questions." )
        )

    no_ratings = schema.List(
        title = _( u'label-question.rating.IRating.no_ratings',
                   u"Alternative options" ),
        description = _( u'help-question.rating.IRating.no_ratings',
                         u"This allows the participant to skip this rating." ),
        required = False,
        default = [],
        value_type = schema.TextLine(
            title = u"No rating option" ),
            )

    base_options = schema.List(
        title = _( u'label-question.rating.IRating.base_options',
                   u"Base options", ),
        description = _( u'help-question.rating.IRating.base_options',
                         u"Enter the values of possible ratings." ),
        value_type = schema.Object(
            title = _( u'label-question.rating.IRating.base_options.value_type',
                       u"Option", ),
            description = _(
                u'help-question.rating.IRating.base_options.value_type',
                u"" ),
            schema = IRatingOption
            )
        )

    # ???
    #active = schema.Bool(
    #    title = _( u'label-question.rating.IRatingTemplate.active',
    #               u"Active" ),
    #    description = _( u'help-question.rating.IRatingTemplate.active',
    #                     u"Check this box if you want to use this rating "
    #                     u"template for your question sheets." ),
    #    default = False )

class IRatingTemplates(Interface):
    """A utility providing available rating templates.
    """

    templates = schema.Iterable(
        title = _( u'label-question.rating.IRatingTemplates.templates',
                   u"Rating templates" ),
        description = _( u'help-question.rating.IRatingTemplates.templates',
                         u"A list of the unit's available rating templates." )
        )



##############################################################################

class IRatingGroup(Interface):
    """A group of rating questions.
    """
    contains('zw.vogon.question.rating.interfaces.IRatingQuestion')



class IRatingResponseSchema(IResponseSchema):
    """
    """

    response = RatingQuestionField( title = u'Response' )
    #not_rated = schema.Bool( title = u'Not rated' )
