#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapts, getUtility
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleVocabulary
from zope.app.interface import PersistentInterfaceClass

from zw.vogon.question.question import Question
from zw.vogon.question.interfaces import IResponseFactory
from zw.vogon.question.response import ResponseContained, ResponseFactory
from zw.vogon.question.rating.interfaces import IRatingQuestion, IRatingResponseSchema
from zw.vogon.question.rating.field import RatingQuestionField
from zw.vogon.question.rating.option import RatingOptionTerm, RatingOptionList
from zw.vogon.question.rating.utils import calculateColorMap

from plone.i18n.normalizer.interfaces import IURLNormalizer

class RatingQuestion(Question):
    implements(IRatingQuestion)

    _base_options = None

    #template = FieldProperty(IRatingQuestion['template'])
    #base_options = FieldProperty(IRatingQuestion['base_options'])

    no_ratings = FieldProperty(IRatingQuestion['no_ratings'])
    base_options = None

    
    def generateOptionsVocabulary(self):
        if not self.base_options:
            return SimpleVocabulary( () )

        base_options = RatingOptionList()
        for option in self.base_options:
            if option.value is not None:
                base_options.append(option)
        base_options.sort()
    
        terms = []
        
        for i in range(len(base_options)-1):
            start, end = base_options[i:i+2]

            values = range(start.value, end.value)
            labels = [start.label] + [ None for i in range( len(values)-1 ) ]
            colors = calculateColorMap( start=start.color, end=end.color,
                                        count=len(values) )
        
            terms += map( lambda v,l,c: RatingOptionTerm( v, label=l, color=c ),
                          values, labels, colors )

        terms.append( RatingOptionTerm( base_options[-1].value,
                                        label=base_options[-1].label,
                                        color=base_options[-1].color ) )

        # It seems to be possible that no_rating is None
        if self.no_ratings is None:
            self.no_ratings = []

        normalizer = getUtility(IURLNormalizer)
        for i, noRating in enumerate(self.no_ratings):
            terms.append( RatingOptionTerm( 
                    noRating, 
                    token = normalizer.normalize(noRating)+'-%i' % i,
                    label = noRating,
                    color = 'cccccc',
                    extra_term = True ) )
                              
        return SimpleVocabulary( tuple(terms) )
    

def updateResponseSchema(question, event):
    """Handler to keep the response_schema up to date.
    """
    question.response_schema = PersistentInterfaceClass(
        name = 'IRatingResponseSchema',
        bases = (IRatingResponseSchema, ),
        attrs = {'response': RatingQuestionField(
                title = u'',
                required = question.required,
                vocabulary = question.generateOptionsVocabulary() )
                 },
        __doc__ = """The concrete response schema.""",
        __module__ = 'zw.vogon.question.rating.interfaces' )

class RatingResponseContained(ResponseContained):

    response = None # FieldProperty(IRatingResponseSchema['response'])


class RatingResponseFactory(ResponseFactory):
    implements(IResponseFactory)
    adapts(IRatingQuestion)

    title = _( u'Rating response factory' )
    description = _( u'This factory instantiates new rating responses.' )
    _callable = RatingResponseContained


    
