#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.interface import Interface

class IRatingQuestionResults(Interface):
    """Computes some values for RatingQuestions.
    """

    optionsMapping = schema.Tuple(
        title = u"An ordered mapping of the options without extra options",
        value_type = schema.Dict(
            title = u"Option details",
            description = u"""{ 
               'value':    value,
               'label':    label option,
               'color':    color,
               'abs': absolute number of answers,
               'rel': relative number of answers, }""",
            ) )

    extraOptionsMapping = schema.Tuple(
        title = u"An ordered mapping of the extra options",
        value_type = schema.Dict(
            title = u"Option detail",
            description = u"""{
                'value': value,
                'abs':   absolute number of answers,
                'rel':   relative number of answers, }""",
            ) )
    
    mean = schema.Float(
        titel = u"Mean value" )

    standardDeviation = schema.Float(
        title = u"Standard deviation" )

    numberOfNoRatings = schema.Int(
        title = u"Number of no_rating responses.",
        min = 0 )

    

class IRatingResultSettings(Interface):
    """Results settings specify some preferences for generating results
    of rating questions.
    """

    discard_empty_options = schema.Bool(
        title = _( u'label-question-rating-results.IRatingResultsSettings-discard_empty_options',
                   u"Discard options without results" ),
        default = False )

