#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from StringIO import StringIO

from zope.cachedescriptors.property import Lazy

from z3c.pagelet.browser import BrowserPagelet

from zw.vogon.question.results.latex import cleanSRC
from zw.vogon.question.results.renderer import HTMLResultRenderer, \
    LaTeXResultRenderer, XLSResultRenderer
from zw.vogon.question.results.results import QuestionResults

class RatingQuestionResults(QuestionResults):

    @Lazy
    def _vocab(self):
        return self.question.generateOptionsVocabulary()

    @Lazy
    def _mapping(self):
        mapping = {}.fromkeys([t.value for t in self._vocab], 0)
        
        for response in self._core_responses:
            if response.response is None:
                continue
            mapping[response.response] += 1
        return mapping

    @Lazy
    def numberOfRatings(self):
        return sum( [count for value, count in self._mapping.items() \
                         if type(value) is int ])
        #return self.count - self.numberOfSkipped - self.numberOfNoRatings


    @Lazy
    def optionsMapping(self):
        if self.numberOfRatings < 1:
            return tuple()
        mapping = self._mapping
        vocab = self._vocab
        return tuple( [ { 'value': term.value,
                          'label': term.label,
                          'color': term.color,
                          'abs': mapping[term.value],
                          'rel': float(mapping[term.value])/self.numberOfRatings,
                          } for term in vocab if not term.extra_term ] )
    
    @Lazy
    def extraOptionsMapping(self):
        if self.numberOfRatings < 1:
            return tuple()
        mapping = self._mapping
        vocab = self._vocab
        return tuple( [ { 'value': term.value,
                          'label': term.label,
                          'color': term.color,
                          'abs': mapping[term.value],
                          'rel': float(mapping[term.value])/self.numberOfRatings,
                          } for term in vocab if term.extra_term ] )
    
    
    
    @Lazy
    def mean(self):
        if self.numberOfRatings < 1:
            return
        return \
            sum([count*float(value) for value, count in self._mapping.items() \
                     if type(value) is int ]) / self.numberOfRatings 

    @Lazy
    def standardDeviation(self):
        if self.numberOfRatings <= 1:
            return
        return pow( sum( 
            [ count*pow(value-self.mean, 2) for value, count in \
                  self._mapping.items() if type(value) is int ] ) \
                        / ( self.numberOfRatings - 1 ), 0.5 )

    @Lazy
    def standardError(self):
        return self.standardDeviation/pow(self.numberOfRatings, 0.5)


    @Lazy
    def numberOfNoRatings(self):
        return sum( [ 1 for r in self._core_responses if r.response in self.context.no_ratings ] )


class RatingQuestionResultsPage(HTMLResultRenderer, BrowserPagelet, \
                                    RatingQuestionResults):

    def __init__(self, context, request):
        super(RatingQuestionResultsPage, self).__init__(context, request)
        RatingQuestionResults.__init__(self, context)


rating_tex = u"""%%%% question: rating
\\noindent\\parbox[t]{.48\\textwidth}{
 \\textbf{%(index)s\\hspace{2mm} %(title)s} ($%(count)i\\times$)
 %(body)s
}
\\medskip"""

rating_extra_options = u"""\\hline
   \\multicolumn{%i}{|l|}{%s}\\\\\\hline"""

rating_body = u"""\\begin{center}
  \\begin{tabular}{|l%(table-options)s|}\\hline
   \\multicolumn{%(count-options-p)i}{|l|}{$\\varnothing$ %(mean)s $\\pm$ %(standard_deviation)s}\\\\[2mm]
   & \\rule{0mm}{20mm}%(color-bars)s \\\\[-3.3pt]\\cline{2-%(count-options-p)i}
   & \\multicolumn{%(count-options)i}{c|}{\\footnotesize \\hspace*{-1mm}%(labels)s \\hspace*{-1mm}}\\\\\\hline
   Note & %(values)s\\\\\\hline
   Stimmen & %(abs-counts)s\\\\\\hline
   %(extra-options)s
  \\end{tabular}
 \\end{center}"""



#   &  & \\frame{\\textcolor[HTML]{99FF32}{\\rule{5mm}{3.933333mm}}} & \\frame{\\textcolor[HTML]{FFFF32}{\\rule{5mm}{1.766667mm}}} & \\frame{\\textcolor[HTML]{FF9932}{\\rule{5mm}{0.100000mm}}} & \\frame{\\textcolor[HTML]{FF3232}{\\rule{5mm}{0.100000mm}}}\\\\[-3.3pt]

rating_bar_tex = u"\\frame{\\textcolor[HTML]{%s}{\\rule{5mm}{%smm}}}"


class RatingQuestionResultsTeX(LaTeXResultRenderer, RatingQuestionResults):

    def __init__(self, context, request):
        super(RatingQuestionResultsTeX, self).__init__(context, request)
        RatingQuestionResults.__init__(self, context)

    def render(self):
        self.update()

        texsrc = StringIO()

        if self.numberOfRatings < 1:
            body = u"\n\\hspace{3mm}\\emph{Keine Antworten}"
        else:
            c_options = len(self.optionsMapping)

            #body = '%% count responses: %i\n' % self.numberOfRatings + rating_body % {
            body = rating_body % {
                'table-options': c_options*u'c',
                'count-options-p': c_options+1,
                'mean': ( self.mean is None ) and '/' or '%.1f' % self.mean,
                'standard_deviation': ( self.standardDeviation is None ) and '/' or '%.2f' % self.standardDeviation,
                'color-bars': u' & '.join(
                    [ rating_bar_tex % ( item['color'].upper(), item['rel']*30. ) 
                      for item in self.optionsMapping ]  
                    ),
                'count-options': c_options,
                'labels': u' \\hfill '.join(
                    [ item['label'] for item in self.optionsMapping if item['label'] ] ),
                'upper-label': u'',
                'values': ' & '.join( 
                    [ str(item['value']) for item in self.optionsMapping ] ),
                'abs-counts': ' & '.join(
                    [ str(item['abs']) for item in self.optionsMapping ] ),
                'extra-options': len(self.extraOptionsMapping)>0 and \
                    rating_extra_options % ( c_options+1, u', '.join(
                        [ u'%(label)s: %(abs)i' % item for \
                              item in self.extraOptionsMapping ] ) ) \
                    or u''
                }
        
        print >> texsrc, rating_tex % {
            'index': self.index,
            'title': cleanSRC(self.context.title),
            'count': self.count,
            'body': body,
            }

        return texsrc.getvalue()



class RatingQuestionResultsXLS(XLSResultRenderer, RatingQuestionResults):
    
    ws = None
    col = 0
    response_mapping = {}

    def __init__(self, context, request):
        super(RatingQuestionResultsXLS, self).__init__(context, request)
        RatingQuestionResults.__init__(self, context)
        
    def update(self):
        self.ws.write( 0, self.col, self.index ) 
        self.ws.write( 1, self.col, self.context.title )
        for r in self._core_responses:
            if r.participant not in self.response_mapping:
                self.response_mapping[r.participant] = len(self.response_mapping)+1
            if r.response is not None:
                self.ws.write( self.response_mapping[r.participant]+2, self.col,
                               r.response )

        self.col += 1
