#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
""" Helper functions
$Id$
"""
__docformat__ = 'reStructuredText'

from colorsys import rgb_to_hsv, hsv_to_rgb


sgn = lambda x: x<0 and -1 or 1

def calculateColorMap(start, end=None, count=1):
    """The map is calculated for 'count' entries without end.
    """
    if end is None:
        end=start

    # convert start and end to HSV
    start_rgb = [ int(start[0:2], 16)/255.,
                  int(start[2:4], 16)/255.,
                  int(start[4:6], 16)/255. ]
    end_rgb = [ int(end[0:2], 16)/255.,
                int(end[2:4], 16)/255.,
                int(end[4:6], 16)/255. ]
    start_hsv = rgb_to_hsv( *start_rgb )
    end_hsv = rgb_to_hsv( *end_rgb )

    step_hsv = map(lambda s,e: (e-s)/count, start_hsv, end_hsv)
    # Go the shortest way on the color wheel
    if abs(end_hsv[0]-start_hsv[0]) > 0.5:
        step_hsv[0] -= float(sgn(step_hsv[0]))/count

    for k in range(count):
        hsv = [ (start_hsv[0] + k*step_hsv[0]) % 1 ] + \
            map(lambda s,step: (s + k*step), start_hsv[1:], step_hsv[1:])
        rgb = hsv_to_rgb( *hsv )
        yield ''.join( [ ( '%02x' % (i*255+0.5) ) for i in rgb ] )

