#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from StringIO import StringIO
from pyExcelerator.Workbook import Workbook

from zope.component import getMultiAdapter, getUtility
from zope.component.interfaces import ComponentLookupError
from zope.app.intid.interfaces import IIntIds
from z3c.pagelet.browser import BrowserPagelet

from zw.vogon.participant.interfaces import IParticipants
from  zw.vogon.survey.interfaces import ISurvey

from zw.vogon.question.results import latex
from zw.vogon.question.results.renderer import HTMLResultRenderer, \
    LaTeXResultRenderer, XLSResultRenderer
from zw.vogon.question.results.interfaces import IQuestionResults

from zw.vogon.template.interfaces import IClone

class SurveyResultsPage(HTMLResultRenderer, BrowserPagelet):
    pass

survey_head = u"""\\thispagestyle{plain}
{\\indent\\today}
\\hfill %(count-feedback)i~von~%(count-total)i~Stimmen~(%(relative-feedback).1f\\%%)\\\\[5mm]
\\noindent\\markboth{\\textsc{%(title)s}}{\\textsc{%(title)s}}
\\noindent\\textbf{\\LARGE %(title)s}\\\\[3mm]
%(attributes)s
\\vspace{16mm}
\\small"""

from zope.app.generations.utility import findObjectsProviding
from zw.vogon.question.interfaces import IResponsable, IResponseFolder

class SurveyResultsTeX(LaTeXResultRenderer):

    def render(self):
        self.update()

        texsrc = StringIO()

        participants = IParticipants(self.context)
 
        # Ultra dirty hack
        #finished = 0
        #for q in findObjectsProviding(self.context, IResponsable):
        #    break
        #finished = IResponseFolder(q).keys()
            
        print >> texsrc, survey_head % {
            'count-feedback': participants.numberOfFinished,
            'count-total': len(participants),
            'relative-feedback': 100.*participants.numberOfFinished/(len(participants)),
            'attributes': u"\\\\".join(
                [ u"{\\large\\indent \\emph{%(title)s}: %(value)s}" % item \
                      for item in self.attributes if item['public'] ] ),
            'title': latex.cleanSRC(self.context.title),
            }
        
        for item in self.subviews:
            print >> texsrc, item.render()

        return texsrc.getvalue()

class SurveyResultsPDF(SurveyResultsTeX):

    content_type = 'application/pdf'
    suffix = '.pdf'

    def __call__(self):
        tex_code = super(SurveyResultsPDF, self).__call__()

        # process
        pdf = latex.latex('/usr/bin/pdflatex', 'pdf', tex_code)
        return pdf

    
class SurveyFolderResultsTeX(LaTeXResultRenderer):
    """This view collects the results of all surveys
    in a survey folder.
    """

    def update(self):
        views = []
        for item in self.context.values():
            if ISurvey.providedBy(item):
                view = getMultiAdapter( (item, self.request), 
                                        name = self.viewname )
                view.update()
                views.append(view)

        self.subviews = tuple(views)

    def render(self):
        self.update()

        texsrc = StringIO()

        for item in self.subviews:
            print >> texsrc, item.render()
            print >> texsrc, u'\\cleardoublepage'
            
        return texsrc.getvalue()
        


class SurveyFolderResultsPDF(SurveyFolderResultsTeX):

    content_type = 'application/pdf'
    suffix = '.pdf'

    def __call__(self):
        tex_code = super(SurveyFolderResultsPDF, self).__call__()

        # process
        pdf = latex.latex('/usr/bin/pdflatex', 'pdf', tex_code)
        return pdf


class SurveyResultsXLS(XLSResultRenderer):

    wb = None

    def __init__(self, context, request):
        super(SurveyResultsXLS, self).__init__(context, request)
        self.wb = Workbook()

    def update(self):
        ws = self.wb.add_sheet(self.context.title)
        col = 0
        response_mapping = {}

        for index, item in enumerate(self.context.values()):
            try:
                view = getMultiAdapter( (item, self.request),
                                        name = self.viewname )
            except ComponentLookupError:
                continue
            view.col = col
            view.ws = ws
            view.response_mapping = response_mapping
            view.indices = [index]
            view.update()
            col = view.col
            

    def render(self):
        self.update()
        return self.wb

class SurveyFolderResultsXLS(XLSResultRenderer):

    wb = None

    def __init__(self, context, request):
        super(SurveyFolderResultsXLS, self).__init__(context, request)
        self.wb = Workbook()


    def update(self):
        for item in self.context.values():
            try:
                view = getMultiAdapter( (item, self.request),
                                        name = self.viewname )
            except ComponentLookupError:
                continue
            view.wb = self.wb
            view.update()
            

    def render(self):
        self.update()
        return self.wb




class SurveyFolderConflatedResultsTeX(LaTeXResultRenderer):
    """This view collects the results of all surveys
    of a give template.

    Option: tid=
    """

    survey_template = object()

    def update(self):
        intids = getUtility(IIntIds)
        try:
            template = intids.queryObject(self.request.get('tid'))
            assert template is not None

        except:
            self.subview = ()
            return

        views = []
        for item in self.context.values():
            try:
                assert IClone(item).template == self.survey_template
            except:
                continue


            
        #    if ISurvey.providedBy(item):
        #        view = getMultiAdapter( (item, self.request), 
        #                                name = self.viewname )
        #        view.update()
        #        views.append(view)

        self.subviews = tuple(views)

    def render(self):
        self.update()

        texsrc = StringIO()

        for item in self.subviews:
            print >> texsrc, item.render()
            print >> texsrc, u'\\cleardoublepage'
            
        return texsrc.getvalue()
        

class SurveyFolderConflatedResultsPDF(SurveyFolderConflatedResultsTeX):

    content_type = 'application/pdf'
    suffix = '.pdf'

    def __call__(self):
        tex_code = super(SurveyFolderConflatedResultsPDF, self).__call__()

        # process
        pdf = latex.latex('/usr/bin/pdflatex', 'pdf', tex_code)
        return pdf
