#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _

import os.path
from tempfile import mkstemp, gettempdir
from os import write, close, \
     chdir, listdir, getcwd, unlink, rmdir, \
     spawnv, waitpid, P_WAIT



TeXHeader = u"""\\documentclass[10pt,twoside,a4paper]{report}

\\usepackage{ucs}
\\usepackage[utf8x]{inputenc}
\\PrerenderUnicode{äüößÄÜÖß'`’}
\\usepackage{multicol}
\\usepackage[german]{babel}
\\usepackage{amssymb}
\\usepackage{eurosym}
\\usepackage[HTML]{xcolor}
\\usepackage{tabularx}
\\usepackage{icomma}

\\setlength{\\textwidth}{17cm}
\\setlength{\\textheight}{240mm}
\\setlength{\\headsep}{5mm}
\\setlength{\\headheight}{5mm}
\\setlength{\\oddsidemargin}{0mm}
\\setlength{\\evensidemargin}{-10mm}
\\setlength{\\topmargin}{-10mm}
\\setlength{\\columnseprule}{0.4pt}
\\setlength{\\columnsep}{25pt}
\\setlength{\\extrarowheight}{1.2pt}
\\setlength{\\tabcolsep}{2ex}

\\renewcommand{\\labelitemi}{$\\circ$}
\\pagestyle{myheadings}
\\raggedbottom
\\raggedcolumns
\\setcounter{chapter}{1}
"""


def cleanSRC(src):
    """Helper to create valid TeX-Code for extra symbols.
    """
    if src is None:
        src = u''
    src = src.replace('\\', '\\backslash')
    src = src.replace('$', '\\$')
    src = src.replace('<', '$<$')
    src = src.replace('>', '$>$')
    src = src.replace('\\backslash', '$\\backslash$')
    src = src.replace('"', "''")
    for char in ('#', '&', '_', '%', '{', '}',):
        src = src.replace(char, '\\'+char)
    for char in ('~', '^',):
        src = src.replace(char, '\\'+char+'{}')
    return src



##########################################################################
### Taken from DTMLTeX by Martin Kelc, gocept GmBH & Co KG
###

errorcolor = "#fc6"

# This is for running the latex-command
def latex(binary, ext, tex_code):
    tempdir = gettempdir()

    handle, texpath = mkstemp('.tex')

    tex = os.path.basename(texpath)
    texbasedot = tex[:-3]
    output = texbasedot + ext
    log = texbasedot + 'log'

    # create temporary tex file
    write(handle, tex_code)
    close(handle)

    rerun = True  # flag for running the command again
    runs = 0      # count of runs already done.

    cwd = getcwd()
    try:
        chdir(tempdir)
        while rerun and runs <= 10:
            rerun = False

            try:
                if spawnv(P_WAIT, binary,
                          (binary, '-interaction=batchmode', tex)):
                    raise 'CommandError'
                runs += 1
            except ('CommandError', OSError):
                # OSError occurs on Win when a file (e.g. image) is not found
                logdata = file(log).readlines()
                raise 'LatexError', logdata

            logdata = file(log).readlines()

            # if the output contains hints about rerunning the
            # generation (content etc) we do so ...
            # but at maximum 10 times ...
            for line in logdata:
                #if line.startswith("LaTeX Warning:") \
                #           and line.lower().find("rerun") != -1:
                if line.lower().find("rerun latex") != -1:
                    rerun = True

                if line == "! Emergency stop." or \
                       line == "No pages of output.":
                    raise 'LatexError', logdata

        out = file(output, "rb").read()
    finally:
        chdir(cwd)

        for i in listdir(tempdir):
            # XXX Is this safe? If a filename created by mkstemp is
            # allowed to contain a dot, this filter is not enough.
            if i.startswith(texbasedot):
                unlink(os.path.join(tempdir, i))

    return out

# The next lines are the Code-o-Beautifier *G
def compose_errmsg(logdata, tex_code):
    # Pick error lines from latex log, mark up log
    errorlines = []
    contline = 0
    errlog = ""
    for line in logdata:
        line = html_quote(line);

        if contline:
            stripped_line = line.lstrip()
            whitespace = line[:len(line)-len(stripped_line)]
            errlog += '%s<a href="#line%s">%s</a>\n</strong>' % \
                (whitespace, errline, stripped_line)
            contline = 0
            continue

        if line[:2] == "! ":
            errlog += "<strong>%s</strong>" % line
            continue

        if line[:2] == "l.":
            errline = None
            try:
                errline = int(line.split(" ")[0][2:])
            except:
                pass

            if errline is not None:
                errorlines.append(errline)
                contline = 1
                errlog += \
                    "<strong><a href=\"#line%s\">%s</a>" \
                    % (errline, line.rstrip())
                continue

        errlog += "%s" % line

    # mark up LaTeX source
    tf = tex_code.split("\n")
    numberwidth = len("%s" % len(tf))
    texlog = ""

    for (lineno, line) in zip(range(1,len(tf)+1),tf):
        line = html_quote("%*d %s" \
                          % (numberwidth, lineno, line))
        if lineno in errorlines:
            texlog += "<strong><a name=\"line%s\">%s</a>\n" \
                "</strong>" % (lineno, line)
        else:
            texlog += "%s\n" % line

    # create output document with error infos
    errmsg = \
r'''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"
                       "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
    <meta http-equiv="content-type"
          content="text/html; charset=ISO-8859-1">
    <title>LaTeX compilation had errors.</title>
    <style type="text/css">
    <!--
        strong {
            display:block;
            color:black;
            background-color:%s;
            }
    </style>
</head>
<body>
    <h1>There were LaTeX errors.</h1>

    <p>
      You receive this output because (pdf)latex was not able to
      compile the .tex file generated by this DTMLTeX object
      correctly. You can find the
      <a href="#latexfile">generated LaTeX file</a> at the bottom of
      this page.
    </p>

    <h2>LaTeX output (log file)</h2>

<pre>
%s</pre>

    <h2><a name="latexfile">Generated LaTeX content</a></h2>

<pre>
%s</pre>

</body>
</html>
''' % (errorcolor, errlog, texlog)

    return errmsg
