#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _

from datetime import datetime
import pytz

from email import Encoders
from email.MIMEBase import MIMEBase
from email.MIMEText import MIMEText
from email.MIMEMultipart import MIMEMultipart

from zope.cachedescriptors.property import Lazy
from zope.component import getMultiAdapter, getUtility
from zope.interface import alsoProvides
from zope.publisher.browser import TestRequest
from zope.schema import getFields
from zope.traversing.api import getName

from zw.vogon.layer import IVogonBrowserLayer
from zw.vogon.mail.settings import IMailSettings
from zw.vogon.survey.interfaces import IStoreSurveyAttributes, \
    ISurveyAttributes
from zw.vogon.survey.mailtext import mergeText
from zw.vogon.question.results.browser.survey_results import SurveyResultsPDF

mailsubject = u"Evaluationsergebnisse"

mailtext_old = u"""Liebe Kolleginnen und Kollegen!

Die Evaluation der Lehre für dieses Semester ist inzwischen abgeschlossen. Anbei erhalten Sie die Auswertung zu Ihrer Veranstaltung „<TITLE>“.

Leider ist der Rücklauf hinter den Erwartungen zurückgeblieben, sodass für einige Veranstaltungen nur relativ kleine Fallzahlen vorliegen, allerdings hat sich das Verfahren technisch bewährt.

In der nächsten Woche werden Sie zudem ein „Zeugnis“ erhalten, dass die Benotung Ihrer Veranstaltung (im Vergleich zum Fakultätsmittelwert) enthalten wird.

Mit freundlichen Grüßen, 
Josef Schmid
"""

mailtext = u"""Liebe Kolleginnen und Kollegen!

Leider ist bei den Evaluierungsergebnissen der Mittelwert in einigen Fällen falsch berechnet worden. Daher erhalten Sie anbei eine korrigierte Version der Ergebnisse.

Zudem möchte ich Ihnen anbieten, dass Veranstaltungen, die nicht evaluiert worden sind oder zum Zeitpunkt der Evaluation noch nicht stattgefunden hatten, nachevaluiert werden können. Die Evaluation würde im Zeitraum vom 28.07.08 bis zum 10.08.08 stattfinden. Bitten melden Sie sich dafür unter lehrevaluation08@gmx.de. 

Mit freundlichen Grüßen,
Josef Schmid 
"""

class MailResults(object):

    context = None
    mailbody = mailtext
    mailsubject = mailsubject
    recipent_attr = 'email'

    def __init__(self, context):
        self.context = context

    def createResultsPDF(self):
        request = TestRequest()
        alsoProvides(request, IVogonBrowserLayer)
        view = SurveyResultsPDF(self.context, request)
        view.update()
        return view()


    @Lazy
    def default_mapping(self):
        """Helper to gets the default mapping, i.e. title, persons, etc.
        """
        mapping = { 'title': self.context.title }
        if IStoreSurveyAttributes.providedBy(self.context):
            attributes = ISurveyAttributes(self.context)
            for name, field in getFields(attributes.__schema__).items():
                mapping[name] = getattr(attributes, name, field.default)
        return mapping


    def composeMessage(self, mapping, sender=None, recipent=None,
                       use_defaults=True):
        if use_defaults:
            _mapping = self.default_mapping
        else:
            _mapping = {}

        for key, item in mapping.items():
            _mapping[key] = item

        # recipent
        if self.recipent_attr in _mapping:
            recipent = _mapping[self.recipent_attr]


        message = MIMEMultipart()
        message.preamble = 'This is a multi-part message in MIME format.'
        message.epilogue = ''
        body_msg = MIMEText( mergeText( self.mailbody,
                                        _mapping ).encode('utf-8'),
                             'plain', 'utf-8' )
        message.attach(body_msg)

        results_file = MIMEBase('application', 'pdf')
        results_file.set_payload(self.createResultsPDF())
        Encoders.encode_base64(results_file)
        results_file.add_header('Content-Disposition', 'attachment', 
                                filename='%s.pdf' % getName(self.context))
        message.attach(results_file)

        message['Subject'] = mergeText( self.mailsubject, _mapping )
        message['Date'] = datetime.now(pytz.utc).strftime('%a, %d %b %Y %H:%M:%S %z')
        if sender is None:
            mail_settings = getUtility(IMailSettings)
            sender = mail_settings.email_from_address
        message['From'] = sender
        if recipent is not None:
            message['To'] = recipent
        return message
