#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _

from os import fdopen
from tempfile import mkstemp
from StringIO import StringIO

from zope.cachedescriptors.property import Lazy
from zope.component import getMultiAdapter
from zope.interface import implements
from zope.publisher.browser import BrowserPage
from zope.schema import getFieldNames
from zope.traversing.api import getName
from z3c.pagelet.browser import BrowserPagelet

from zw.vogon.question.interfaces import IQuestionContainer
from zw.vogon.survey.interfaces import ISurveyAttributes, ISheetContainer

from zw.vogon.question.results import latex
from zw.vogon.question.results.interfaces import IResultRenderer, \
    IHTMLResultRenderer, IXLSResultRenderer, ILaTeXResultRenderer, \
    IQuestionResults

class ResultRenderer(BrowserPage):
    implements(IResultRenderer)

    viewname = None
    results = None
    indices = []
    subviews = ()
    skip_indexing = False

    @property
    def index(self):
        return self.indices and u'.'.join( [unicode(i+1) for i in self.indices ] )+'.' or u''

    
    @Lazy
    def settings(self):
        return IResultSettings(self.context, None)


    @property
    def attributes(self):
        attributes = ISurveyAttributes(self.context, None)
        if attributes is None:
            return []

        items = []
        schema = attributes.__schema__
        for name in getFieldNames(schema):
            value = getattr(attributes, name, None)
            if value:
                items.append(
                    { 'title': schema[name].title,
                      'value': value,
                      'public': getattr(schema[name], 'public', True) } )
        return items


    def update(self):
        try:
            super(ResultRenderer, self).update()
        except AttributeError:
            pass

        self.results = IQuestionResults(self.context, None)

        assert self.viewname is not None, \
            "Subclasses must provide the 'viewname' attribute."
        if IQuestionContainer.providedBy(self.context) or \
                ISheetContainer.providedBy(self.context):
            views = []
            c_skipped = 0
            for index, (name, item) in enumerate(self.context.items()):
                view = getMultiAdapter( (item, self.request), 
                                        name = self.viewname )
                view.indices = self.indices + [index-c_skipped]
                if view.skip_indexing:
                    c_skipped += 1
                view.update()
                views.append(view)
                
            self.subviews = tuple(views)

    def render(self):
        try:
            return super(ResultRenderer, self).render()

        except AttributeError:
            raise NotImplementedError


class HTMLResultRenderer(ResultRenderer):
    implements(IHTMLResultRenderer)

    viewname = u'results.html'


class RawResultRenderer(ResultRenderer):

    viewname = u'results.csv'    

class XLSResultRenderer(RawResultRenderer):
    implements(IXLSResultRenderer)

    viewname = u'results.xls'
    content_type = 'application/vnd.ms-excel'
    suffix = '.xls'

    def __call__(self):
        filename = getName(self.context) + self.suffix
        response = self.request.response
        response.setHeader( 'Content-Disposition',
                           ' attachment; filename=%s' % filename )
        response.setHeader( 'Content-Type', self.content_type )

        handle, xlspath = mkstemp(self.suffix)
        wb = self.render()
        wb.save(xlspath)

        xls = fdopen(handle)
        return xls.read()
        


class LaTeXResultRenderer(ResultRenderer):
    implements(ILaTeXResultRenderer)

    viewname = u'results.tex'
    content_type = 'application/x-latex'
    suffix = '.tex'

    def __call__(self):
        filename = getName(self.context) + self.suffix
        response = self.request.response
        response.setHeader( 'Content-Disposition',
                           ' attachment; filename=%s' % filename )
        response.setHeader( 'Content-Type', self.content_type )

        texsrc = StringIO()
        
        print >> texsrc, latex.TeXHeader
        print >> texsrc, u'\\begin{document}'

        print >> texsrc, self.render()
         
        print >> texsrc, u'\\end{document}'
        return texsrc.getvalue().encode('utf-8')



