#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapts
from zope.interface import implements, Interface
from zope.traversing.api import getParent, getName

from z3c.form import button, field, form

from zw.vogon.question.browser.form import ItemGroupForm, \
    ManageItemGroupForm, AddItemForm

from zw.vogon.question.sheet.browser.interfaces import ISheetForm, \
    IManageSheetForm, ISheetFrame, IManageSheetFrame, IManageSheetButtons
from zw.vogon.question.sheet.interfaces import ISheet

class SheetForm(ItemGroupForm):
    implements(ISheetForm)
    adapts(ISheet, Interface, ISheetFrame)

    fields = field.Fields(ISheet).omit('__name__', '__parent__')


class ManageSheetForm(SheetForm, ManageItemGroupForm):
    implements(IManageSheetForm)
    adapts(ISheet, Interface, IManageSheetFrame)
    form.extends(SheetForm, ManageItemGroupForm)

    def update(self):
        self.buttons = button.Buttons(IManageSheetButtons) + \
            self.buttons.omit('up', 'down')
        super(ManageSheetForm, self).update()
        
        self.itemActions = [ item for name, item in self.actions.items()
                             if name in self.buttons.omit('save', 'cancel_edit').keys() ]
        self.groupActions = [ item for name, item in self.actions.items()
                              if name not in self.buttons.omit('save', 'cancel_edit').keys() ]


    @button.handler( IManageSheetButtons['add_sheet'] )
    def handleAddSheet(self, action):
        self.session['add'][self.parentForm.prefix] = u'zw.vogon.question.Sheet'
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageSheetButtons['left'] )
    def handleLeft(self, action):
        parentForm = getParent(self)
        parentForm.itemsUp( getName(self.context) )
        self.request.response.redirect(self.request.getURL())

    @button.handler( IManageSheetButtons['right'] )
    def handleRight(self, action):
        parentForm = getParent(self)
        parentForm.itemsDown( getName(self.context) )
        self.request.response.redirect(self.request.getURL())


class AddSheetForm(AddItemForm):

    factory_name = u'zw.vogon.question.Sheet'
    fields = field.Fields(ISheet).omit('__name__', '__parent__')
    
    label = _( u"Add a question sheet" )

    def render(self):
        if self._finishedAdd:
            del self.parentForm.session['add'][self.parentForm.prefix]
            self.request.response.redirect(self.request.getURL()+'?pos=-1')
            return ""
        return super(AddItemForm, self).render()
