#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.cachedescriptors.property import Lazy
from zope.component import getMultiAdapter
from zope.interface import implements
from zope.session.interfaces import ISession
from zope.traversing.api import getPath, getName

from z3c.batching import batch
from z3c.form import button
from z3c.form.interfaces import DISPLAY_MODE, INPUT_MODE
from z3c.formui import form

from zw.vogon.question.sheet.browser.interfaces import ISheetFrame, \
    IManageSheetFrame, ISheetForm, IManageSheetForm
from zw.vogon.question.sheet.browser.sheet_form import AddSheetForm

from zw.vogon.utils.browser.cssbutton import CSSButton

_marker = object()


def handleFormSubmit(form, action):
    print "Form submitted"


def handleNewPos(form, action):
    form.session['container_pos'] = int(action.value)-1
    form.request.response.redirect(form.request.getURL())



class SheetFrame(form.Form):
    implements(ISheetFrame)

    mode = DISPLAY_MODE
    sheet = None
    itemform_mode = 'view'

    @Lazy
    def prefix(self):
        return str(getName(self.getContent())+'.')

    @Lazy
    def label(self):
        return getattr( self.getContent(), 'title', u'' )


    @Lazy
    def sessionKey(self):
        try:
            return 'zw.vogon.survey:%s' % getPath(self.getContent())
        except TypeError:
            return 'zw.vogon.survey:'

    @Lazy
    def session(self):
        session = ISession(self.request)[self.sessionKey]
        if session.get('container_pos', _marker) is _marker:
            session['container_pos'] = 0
        if session.get('add', _marker) is _marker:
            session['add'] = {}
        return session

    @Lazy
    def batch(self):
        c_pos = self.session['container_pos']
        if len(self.getContent().keys()) <= c_pos:
            c_pos -= 1
        if c_pos < 0:
            c_pos = 0
        self.session['container_pos'] = c_pos
        return batch.Batch(self.getContent().values(),
                           start=c_pos, size=1)

    @button.buttonAndHandler( _( u'Previous', ), name='prev', 
                              condition = lambda form: \
                                  form.batch.previous is not None )
    def handlePrev(self, action):
        self.session['container_pos'] -= 1
        self.request.response.redirect(self.request.getURL())

    @button.buttonAndHandler( _( u'Next', ), name='next', 
                              condition = lambda form: \
                                  form.batch.next is not None )
    def handleNext(self, action):
        self.session['container_pos'] += 1
        self.request.response.redirect(self.request.getURL())

    @button.buttonAndHandler( _( u'Finish' ), name='finish',
                              condition = lambda form: \
                                  form.batch.next is None and \
                                  form.itemform_mode is 'view' )
    def handleFinish(self, action):
        pass
        

    def update(self):
        # Allow to overwrite container_pos by request
        try:
            pos = int(self.request.get('pos', None))
        except:
            pass
        else:
            if pos >= 0:
                self.session['container_pos'] = min( 
                    pos, len(self.getContent().keys())-1 )
            else:
                self.session['container_pos'] = max( 
                    0, len(self.getContent().keys())+pos )

        for n, batch in enumerate(self.batch.batches):
            if batch[:]:
                sheet = batch[0]
                if batch == self.batch:
                    sheet_button = CSSButton( 
                        str('%i.%s' % (n, sheet.__name__) ),
                        klass = u'current-sheet',
                        title = unicode(n+1) ) 
                else:
                    sheet_button = button.Button( 
                        str( '%i.%s' % (n, sheet.__name__) ),
                        title = unicode(n+1) ) 
                self.buttons += button.Buttons( sheet_button )
                self.handlers.addHandler( sheet_button, handleNewPos )

        fix_button = button.ImageButton( 
            name='submitfix', title = u'submitfix', 
            image = u'zw.contentsform/transparent-1x1.gif' )
        self.buttons += button.Buttons( fix_button )
        self.handlers.addHandler( fix_button, handleFormSubmit )

        super(SheetFrame, self).update()

        # update action groups 
        self.bottomActions = [ item for name, item in self.actions.items()
                               if name in ('prev', 'next', 'finish') ]
        self.sheetActions = [ item for name, item in self.actions.items()
                               if name not in ( 
                'submitfix', 'prev', 'next', 'finish') ]
                    
        
        if self.batch[:]:
            self.sheet = getMultiAdapter( (self.batch[0], self.request, self),
                                          ISheetForm, name=self.itemform_mode )
            self.sheet.indices = [ self.batch.index ]
            self.sheet.update()
            


class ManageSheetFrame(SheetFrame):
    implements(IManageSheetFrame)

    itemform_mode = 'manage'

    def update(self):
        super(ManageSheetFrame, self).update()

        if not self.batch[:]:
            self.session['add'][self.prefix] = u'zw.vogon.question.Sheet'
            

        if self.adding:
            factory_name = self.session['add'].get(self.prefix)
            item_add_form = AddSheetForm( self.getContent(), self.request, self )
            item_add_form.update()
            self.sheet = item_add_form
            

    @Lazy
    def adding(self):
        return self.session['add'].get(self.prefix) is not None


    # Ordering facilities
    
    def itemsUp(self, *names):
        # check if actual sheet is moved up
        if self.batch[:]:
            if getName(self.batch[0]) in names and \
                    self.batch.previous is not None:
                self.session['container_pos'] -= 1
                
        ind_keys = []
        for i, key in enumerate(self.getContent().keys()):
            i *= 2
            if key in names:
                i -= 3
            ind_keys.append( (i,key) )
        ind_keys.sort()
        self.getContent().updateOrder( tuple( map( lambda i: i[1], ind_keys ) ) )
                
    def itemsDown(self, *names):
        # check if actual sheet is moved up
        if self.batch[:]:
            if getName(self.batch[0]) in names and \
                    self.batch.next is not None:
                self.session['container_pos'] += 1
                
        ind_keys = []
        for i, key in enumerate(self.getContent().keys()):
            i *= 2
            if key in names:
                i += 3
            ind_keys.append( (i,key) )
        ind_keys.sort()
        self.getContent().updateOrder( tuple( map( lambda i: i[1], ind_keys ) ) )


