#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'


from zw.vogon.i18n import MessageFactory as _

import cgi
from StringIO import StringIO

from zope.cachedescriptors.property import Lazy
from z3c.pagelet.browser import BrowserPagelet

from zw.vogon.question.results.latex import cleanSRC
from zw.vogon.question.results.renderer import HTMLResultRenderer, \
    LaTeXResultRenderer
from zw.vogon.question.results.results import QuestionResults

class TextQuestionResults(QuestionResults):
    
    @Lazy
    def uniqueResponses(self):
        return set( [ r.response for r in self._core_responses if r.response is not None ] )

    @Lazy
    def countOfResponses(self):
        mapping = {}.fromkeys(self.uniqueResponses.union([None]), 0)
        for r in self._core_responses:
            mapping[r.response] += 1
            
        return mapping


class TextQuestionResultsPage(HTMLResultRenderer, BrowserPagelet, \
                                  TextQuestionResults):

    def __init__(self, context, request):
        super(TextQuestionResultsPage, self).__init__(context, request)
        TextQuestionResults.__init__(self, context)

    @property
    def renderedResponses(self):
        return tuple( [
                { 'rendered': cgi.escape(response.strip()).replace('\n', '<br />\n'),
                  'count': self.countOfResponses[response] } for response \
                    in self.uniqueResponses ] )


text_tex = u"""%%%% question: itemize
 \\noindent\\textbf{%(index)s\\hspace{2mm} %(title)s} ($%(count)i\\times$)\\\\[-3mm]
  %(text-responses)s
\\medskip"""

textline_tex = u"""%%%% question: itemize
\\noindent\\parbox[t]{.48\\textwidth}{
 \\noindent\\textbf{%(index)s\\hspace{2mm} %(title)s} ($%(count)i\\times$)\\\\[-3mm]
 \\noindent\\hspace*{-2.45mm}\\parbox[t]{.48\\textwidth}{
 %(text-responses)s
  }
}
\\medskip"""

class TextQuestionResultsTeX(LaTeXResultRenderer, TextQuestionResults):

    def __init__(self, context, request):
        super(TextQuestionResultsTeX, self).__init__(context, request)
        TextQuestionResults.__init__(self, context)

    def render(self):
        self.update()

        texsrc = StringIO()

        if len(self.uniqueResponses) > 0:
            responses = '\n'.join(
                [ u'\\item %s%s' % \
                      ( cleanSRC(response),
                        self.countOfResponses[response]>1 and u' (%i)' % \
                            self.countOfResponses[response] or u'' ) \
                      for response in self.uniqueResponses ] )
            responses = u"""\\begin{itemize}\\setlength{\\itemsep}{0pt\\setlength{\\topsep}{0pt}}
   %s
   \\end{itemize}""" % responses

        else:
            responses = u"\n\\hspace{3mm}\\emph{Keine Antworten}"
        
        
        if self.context.rows == 1:
            print >> texsrc, textline_tex % {
                'index': self.index,
                'title': cleanSRC(self.context.title),
                'count': self.count,
                'text-responses': responses,
                }
        else:
            print >> texsrc, text_tex % {
                'index': self.index,
                'title': cleanSRC(self.context.title),
                'count': self.count,
                'text-responses': responses,
                }

        return texsrc.getvalue()

