#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapts
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.app.container.ordered import OrderedContainer
from zope.app.interface import PersistentInterfaceClass

from zw.vogon.question.question import Question
from zw.vogon.question.interfaces import IResponseFactory
from zw.vogon.question.response import ResponseContained, ResponseFactory
from zw.vogon.question.text.interfaces import ITextQuestion, ITextResponseSchema
from zw.vogon.question.text.field import TextQuestionField


class TextQuestion(Question):
    implements(ITextQuestion)

    cols = FieldProperty( ITextQuestion['cols'] )
    rows = FieldProperty( ITextQuestion['rows'] )


def updateResponseSchema(question, event):
    """Handler to keep the response_schema up to date.
    """
    question.response_schema = PersistentInterfaceClass(
        name = 'ITextResponseSchema',
        bases = (ITextResponseSchema, ),
        attrs = { 
            'response': TextQuestionField(
                title = u'',
                required = question.required,
                rows = question.rows,
                cols = question.cols )
            },
        __doc__ = """The concrete response schema.""",
        __module__ = 'zw.vogon.question.text.interfaces' )



class TextResponseContained(ResponseContained):

    response = None # FieldProperty(ITextResponseSchema['response'])


class TextResponseFactory(ResponseFactory):
    implements(IResponseFactory)
    adapts(ITextQuestion)

    title = _( u'Text response factory' )
    description = _( u'This factory instantiates new text responses.' )
    _callable = TextResponseContained
