#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.annotation.interfaces import IAnnotatable
from zope.interface import Attribute, Interface


class IResults(Interface):
    """Get all relevant data from an abstract item,
    so that a corresponding renderer can work.
    """

class IQuestionResults(IResults):
    """To go to the question package...
    """

    count = schema.Int(
        title = u"Number of responses",
        min = 0 )

    numberOfSkipped = schema.Int(
        title = u"Number of skipped responses",
        min = 0 )


    

    
##############################################################################

class IStoreResultSettings(IAnnotatable):
    """Marker to allow storing of annotations to a question.
    """

class IResultSettings(Interface):
    """Results settings specify some preferences for generating results.
    """

    
##############################################################################

class IResultRenderer(Interface):
    """Renders the results of a question.
    """

    indices = schema.List(
        title = u"Indices",
        description = u"List of indices",
        value_type = schema.Int(
            title = u"Index",
            min = 0 ),
        )

    settings = schema.Object(
        title = u"Results settings",
        schema = IResultSettings )

    results = schema.Object(
        title = u"Results",
        schema = IQuestionResults )

    viewname = schema.TextLine(
        title = u"Name of the view adapter" )


    subviews = schema.Tuple(
        title = u"Sub views",
        description = u"List a renderers of contained items.",
        )
    

    def update():
        """Prepare the renderer.
        """

    def render():
        """Renders the results.
        """

class IHTMLResultRenderer(IResultRenderer):
    """Renders results to HTML.
    """

    
class IXLSResultRenderer(IResultRenderer):
    """Renders results to Excel.
    """

class ILaTeXResultRenderer(IResultRenderer):
    """Renders results to LaTeX.
    """

