#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import queryMultiAdapter
from zope.traversing.api import getParent
from zope.viewlet.viewlet import ViewletBase
from zope.app.pagetemplate import ViewPageTemplateFile

from zw.vogon.settings.interfaces import ISettings, ISettingsRoot

class NavigationViewlet(ViewletBase):
    """Category view
    """

    template = ViewPageTemplateFile('navigation.pt')

    def update(self):
        super(NavigationViewlet, self).update()
        
        self.isRoot = ISettingsRoot.providedBy(self.context)

        path = []
        parent = self.context
        while ISettings.providedBy(parent):
            path.insert(0, parent)
            parent = getParent(parent)
            
        self.root, path = path[0], path
        self.data = self._process(self.root, path)
        

    def _process(self, context, path, level=1):
        if path:
            data = []
            for name, item in context.items():
                if not ISettings.providedBy(item):# or \
                        #not item.isAvailable():
                    continue

                info = { 'name':      name,
                         'title':     item.__title__,
                         'icon':      queryMultiAdapter( 
                        (item, self.request), name='zmi_icon'),
                         'items':     (),
                         'selected':  False,
                         'settings':  item,
                         'level':     level }

                if item.__name__ == path[0].__name__:
                    info['items'] = self._process(item, path[1:], level+1)

                if item.__name__ == self.context.__name__:
                    info['selected'] = True
                    info['items'] = self._process(item, [item], 
                                                  level+1)

                data.append( (item.__title__, info) )

            data.sort()
            data = [info for t, info in data]
            return data

    def render(self):
        return self.template(self)
                         
        
