#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.component.interfaces import IObjectEvent
from zope.deprecation import deprecated
from zope.interface import Interface
from zope.app.container.interfaces import IContained, IContainer
from zope.app.container.constraints import containers, contains

from zw.vogon.layer import IVogonBrowserLayer


class ISettingsBrowserLayer(IVogonBrowserLayer):
    """The ``settings'' layer.
    """


class ISettings(IContained):
    """Local utitility storing the portal's settings.
    """
    containers('zope.app.component.interfaces.ISiteManagementFolder')

    __title__ = schema.TextLine(
        title = _( u'label-settings.interfaces.ISettings.title',
                   u"Title" ),
        description = _( u'help-settings.interfaces.ISettings.title',
                         u"" ),
        required = True )

    __description__ = schema.TextLine(
        title = _( u'label-settings.interfaces.ISettings.description',
                   u"Description" ),
        description = _( u'help-settings.interfaces.ISettings.description',
                         u"" ),
        required = False )
    
    __schema__ = schema.InterfaceField(
        title = _( u'label-settings.interfaces.ISettings.schema',
                   u"Schema" ),
        description = _( u'help-settings.interfaces.ISettings.schema',
                         u"" ),
        default = Interface,
        required = True )


class ISettingsContainer(IContainer):
    """Store sub settings.
    """
    contains('zw.vogon.settings.interfaces.ISettings')

class ISettingsRoot(Interface):
    """A portal's settings root folder.
    """
    

class IUpdateSettingsEvent(IObjectEvent):
    """Notify all settings to update.
    """

class ILocalDataStorage(IContained):
    """Instead of z3ext.controlpanel.storage.DataStorage this stores 
    the data in a local utility.
    """

ILocalDataStorage = deprecated( ILocalDataStorage,
                                "Don't use ist anymore..." )
    
