#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import sys

from persistent import Persistent

from zope.component import adapts, adapter, getUtility
from zope.component.interfaces import ObjectEvent
from zope.event import notify
from zope.interface import implements, implementsOnly
from zope.lifecycleevent import ObjectCreatedEvent, ObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.schema import getFieldNames
from zope.schema.fieldproperty import FieldProperty
from zope.app.component.hooks import getSite
from zope.app.container.btree import BTreeContainer
from zope.app.container.contained import ObjectAddedEvent

from zw.vogon.portal.interfaces import INewPortalEvent, IPortal
from zw.vogon.settings.interfaces import ISettings, ISettingsContainer, \
    ISettingsRoot, IUpdateSettingsEvent

class SettingsMixin(object):
    implements(ISettings)

    __title__ = FieldProperty( ISettings['__title__'] )
    __description__ = FieldProperty( ISettings['__description__'] )
    __schema__ = FieldProperty( ISettings['__schema__'] )


class Settings(BTreeContainer):
    implements(ISettings, ISettingsContainer)

    __title__ = FieldProperty( ISettings['__title__'] )
    __description__ = FieldProperty( ISettings['__description__'] )
    __schema__ = FieldProperty( ISettings['__schema__'] )

class SettingsRoot(Settings):
    implements(ISettingsRoot)

class UpdateSettingsEvent(ObjectEvent):
    implementsOnly(IUpdateSettingsEvent)

    def __init__(self, settings, portal):
        super(UpdateSettingsEvent, self).__init__(settings)
        self.portal = portal

@adapter(INewPortalEvent)
def addPortalsSettings(event):
    portal = event.object
    sm = portal.getSiteManager()
    
    settings = SettingsRoot()
    notify(ObjectCreatedEvent(settings))
    portal['settings'] = settings
    notify(ObjectAddedEvent(settings))
    sm.registerUtility( settings, ISettings )

    notify(UpdateSettingsEvent(settings, portal))

class UpdateSettingsHandler(object):
    adapts(IUpdateSettingsEvent) 

    def __init__(self, klass, iface, name):
        self.klass = klass
        self.iface = iface
        self.name = name

    def __call__(self, event):
        root_settings = event.object
        sm = event.portal.getSiteManager()

        if self.name in root_settings:
            settings = root_settings[self.name]
        else:
            settings = self.klass()
            notify(ObjectCreatedEvent(settings))
            root_settings[self.name] = settings
            notify(ObjectAddedEvent(settings))
        
        r_ifaces = [ r.provided for r in sm.registeredUtilities() if \
                         r.component == settings ]
        if ISettings not in r_ifaces:
            sm.registerUtility( settings, ISettings, name=self.name )
        if self.iface not in r_ifaces:
            sm.registerUtility( settings, self.iface )


@adapter(ISettings, IObjectModifiedEvent)
def portalModified(settings, event):
    """Notify the portal about the change.
    """
    portal = getSite()
    notify(ObjectModifiedEvent(portal))
