#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2009 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import sys

from zope import interface
from zope.component.zcml import handler as _handler
from zope.configuration import fields
from zope.configuration.exceptions import ConfigurationError
from zope.schema import getFieldNames
from zope.schema.fieldproperty import FieldProperty
from zope.security import checkPermission
from zope.security.zcml import Permission
from zope.security.checker import Checker, CheckerPublic
from zope.app.component.contentdirective import ClassDirective

from zw.vogon.settings.interfaces import IUpdateSettingsEvent, ISettings
from zw.vogon.settings.settings import Settings, SettingsMixin, \
    UpdateSettingsHandler

class ISettingsDirective(interface.Interface):
    
    name = fields.PythonIdentifier(
        title = u"Name",
        description = u"Name of the settings.",
        required = True )

    schema = fields.GlobalInterface(
        title = u"Settings schema",
        description = u"This attribute specifies the settings schema.",
        required = True )

    title = fields.MessageID(
        title = u"Title",
        description = u"Title of the settings used in UIs.",
        required = True )

    description = fields.MessageID(
        title = u"Description",
        description = u"Description of the settings used in UIs.",
        required = False )

    class_ = fields.GlobalObject(
        title = u"Class",
        description = u"Custom settings class",
        required = False )
        
    permission = Permission(
        title = u"Permission",
        description = u"Default access permission",
        required = False )

    tests = fields.Tokens(
        title = u"Tests",
        description = u"Tests for check availability",
        value_type = fields.GlobalObject(),
        required = False )

    
class SettingsDirective(ClassDirective):

    def __init__(self, _context, name, schema, title,
                 description='', class_=None, 
                 permission=u'zw.vogon.editSettings', tests=()):

        # 1 Create Settings class in the schema's module
        if class_ is not None:
            __module__ = class_.__module__
        else:
            __module__ = schema.__module__

        name = str(name)
        dct = { '__module__': __module__,
                '__title__': title,
                '__description__': description,
                '__schema__': schema }
        for fname in getFieldNames(schema):
            dct[fname] = FieldProperty( schema[fname] )

        if class_ is None:
            SettingsClass = type(name, (Settings,), dct)
        else:
            SettingsClass = type(name, (class_, SettingsMixin,), dct)
        interface.classImplements(SettingsClass, schema)
        setattr( sys.modules[__module__], name, SettingsClass )

        super(SettingsDirective, self).__init__(_context, SettingsClass)

        
        # permissions
        self.require(_context, like_class=Settings)
        self.require(_context,
                     permission = u'zw.vogon.viewSettings',
                     interface = (schema,))
        self.require(_context,
                     permission=u'zw.vogon.editSettings',
                     set_schema = (schema,))
        
        # Add event handler 
        _context.action(
            discriminator = None,
            callable = _handler,
            args = ('registerHandler',
                    UpdateSettingsHandler(SettingsClass, schema, name),
                    (IUpdateSettingsEvent,), u'', _context.info),
            )
                    
