#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from persistent.dict import PersistentDict

from zope import schema
from zope.schema.interfaces import ITextLine
from zope.annotation.interfaces import IAnnotations
from zope.component import adapts
from zope.interface import alsoProvides, classImplements, implements

from zw.vogon.utils.mapping import AnnotationsMapperClass, MappingFieldProperty
from zw.vogon.survey.interfaces import IStoreSurveyAttributes, \
    IBaseSurveyAttributesSchema, ISurveyAttributes

KEY = 'zw.vogon.survey.attributes'

class IAttributeField(ITextLine):
    """An attribute field is a TextLine with a public entry
    """

    public = schema.Bool(
        title = _( u"Public attribute" ),
        required = True,
        default = True )

class AttributeField(schema.TextLine):
    
    def __init__(self, *args, **kw):
        self.public = kw.pop('public')
        super(AttributeField, self).__init__(*args, **kw)

class SurveyAttributes(object):
    implements(ISurveyAttributes)
    adapts(IStoreSurveyAttributes)

    KEY = KEY

    __schema__ = MappingFieldProperty(ISurveyAttributes['__schema__'])

def surveyAttributesFactory(survey, schema=None):
    # Get schema

    annotations = IAnnotations(survey)
    if schema is None:
        schema = annotations[KEY].get('__schema__',
                                      IBaseSurveyAttributesSchema )
    else:
        annotations[KEY] = PersistentDict()
        annotations[KEY]['__schema__'] = schema

    CustomSurveyAttributes = AnnotationsMapperClass(
        name = 'CustomSurveyAttributes',
        KEY = KEY,
        schema = schema,
        class_ = SurveyAttributes,
        __doc__ = """The customized SurveyAttributes.""",
        __module__ = 'zw.vogon.survey.attribute' )
    classImplements(CustomSurveyAttributes, schema)

    return CustomSurveyAttributes(survey)

alsoProvides(surveyAttributesFactory, ISurveyAttributes)
    
