#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from datetime import datetime
import pytz
from email.MIMEText import MIMEText

from zope.annotation.interfaces import IAnnotations
from zope.interface import implements
from zope.component import adapts, getUtility
from zope.cachedescriptors.property import Lazy
from zope.schema import getFields

from zw.vogon.mail.settings import IMailSettings
from zw.vogon.utils.mapping import AnnotationsMapper, MappingFieldProperty

from zw.vogon.survey.interfaces import IStoreSurveyAttributes, ISurveyAttributes
from zw.vogon.survey.interfaces import IMailText, IStoreMailTexts, \
    IInvitationText, IReminderText

def mergeText(message, mapping):
    for key, item in mapping.items():
        if isinstance( item, str ):
            fitem = unicode( item )
        elif isinstance( item, unicode ):
            fitem = item
        elif isinstance( item, list ) or isinstance( item, tuple ):
            fitem = u', '.join(item)
        elif item is None:
            # Ignore if not availble
            continue
        else:
            raise TypeError( 'Unrecognized argument type', item )

        message = message.replace( u'<'+key.upper()+'>', fitem )

    return message


class MailText(AnnotationsMapper):
    """Base class for invitation and reminder mail text classes.
    """
    implements(IMailText)

    #subject = MappingFieldProperty(IMailText['subject'])
    #body = MappingFieldProperty(IMailText['body'])

    @Lazy
    def default_mapping(self):
        """Helper to gets the default mapping, i.e. title, persons, etc.
        """
        mapping = { 'title': self.context.title }
        if IStoreSurveyAttributes.providedBy(self.context):
            attributes = ISurveyAttributes(self.context)
            for name, field in getFields(attributes.__schema__).items():
                mapping[name] = getattr(attributes, name, field.default)
        return mapping
    

    def composeMessage(self, mapping, sender=None, recipent=None,
                       use_defaults=True):
        if use_defaults:
            _mapping = self.default_mapping
        else:
            _mapping = {}

        for key, item in mapping.items():
            _mapping[key] = item

        message = MIMEText( mergeText( self.mapping['body'],
                                       _mapping ).encode('utf-8'),
                            'plain', 'utf-8' )
        message['Subject'] = mergeText( self.mapping['subject'], _mapping )
        message['Date'] = datetime.now(pytz.utc).strftime('%a, %d %b %Y %H:%M:%S %z')
        if sender is None:
            mail_settings = getUtility(IMailSettings)
            sender = mail_settings.email_from_address
        message['From'] = sender
        if recipent is not None:
            message['To'] = recipent
        return message


class InvitationText(MailText):
    implements(IInvitationText)
    adapts(IStoreMailTexts)

    KEY = 'zw.vogon.survey.mailtext.invitation'

    subject = MappingFieldProperty(IInvitationText['subject'])
    body = MappingFieldProperty(IInvitationText['body'])


class ReminderText(MailText):
    implements(IReminderText)
    adapts(IStoreMailTexts)

    KEY = 'zw.vogon.survey.mailtext.reminder'

    subject = MappingFieldProperty(IReminderText['subject'])
    body = MappingFieldProperty(IReminderText['body'])
