#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2008 Gregor Giesen <vogon@zaehlwerk.net>                  #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from persistent import Persistent

from zope.annotation.factory import factory
from zope.component import adapts, adapter
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty
from zope.app.container.contained import Contained, uncontained
from zope.app.container.interfaces import IObjectRemovedEvent
from zope.app.container.ordered import OrderedContainer
from zope.app.interface import PersistentInterfaceClass

from zw.vogon.survey.attribute import AttributeField
from zw.vogon.survey.interfaces import IBaseSurveyAttributesSchema
from zw.vogon.template.interfaces import IStoreSurveyAttributeTemplates, \
    ISurveyAttributeTemplates, ISurveyAttributeTemplateContainer, \
    ISurveyAttributeTemplate


class SurveyAttributeTemplateContainer(OrderedContainer):
    implements(ISurveyAttributeTemplates, ISurveyAttributeTemplateContainer)
    adapts(IStoreSurveyAttributeTemplates)

    def getSurveyAttributesSchema(self):
        attrs = {}
        for name, item in self.items():
            attrs[str(name)] = AttributeField(
                title = item.title,
                description = item.description,
                required = item.required,
                public = item.public,
                readonly = item.readonly,
                default = item.default )
        return PersistentInterfaceClass(
            name = 'ISurveyAttributesSchema',
            bases = (IBaseSurveyAttributesSchema, ),
            attrs = attrs,
            __doc__ = """The customized survey attributes schema.""",
            __module__ = 'zw.vogon.survey.attribute' )
    

attributeFolderFactory = factory(SurveyAttributeTemplateContainer, 
                  'survey_attribute_templates')

class SurveyAttributeTemplate(Contained, Persistent):
    implements(ISurveyAttributeTemplate)

    
    title = FieldProperty(ISurveyAttributeTemplate['title'])
    description = FieldProperty(ISurveyAttributeTemplate['description'])
    required = FieldProperty(ISurveyAttributeTemplate['required'])
    public = FieldProperty(ISurveyAttributeTemplate['public'])
    default = FieldProperty(ISurveyAttributeTemplate['default'])
    readonly = FieldProperty(ISurveyAttributeTemplate['readonly'])
    
@adapter(IStoreSurveyAttributeTemplates, IObjectRemovedEvent)
def removeAttributes(template, event):
    folder = ISurveyAttributeTemplates(template)
    for name, item in folder.items():
        uncontained(item, folder, name)
