#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

import base64

from zope.cachedescriptors.property import Lazy
from zope.component import createObject, queryUtility, queryAdapter, \
    getUtility, queryMultiAdapter
from zope.event import notify
from zope.interface import alsoProvides, implements
from zope.schema import TextLine, getFields
from zope.traversing.browser import absoluteURL
from zope.app.container.constraints import checkObject
from zope.app.container.interfaces import INameChooser
from zope.app.generations.utility import findObjectsProviding
from zope.app.intid.interfaces import IIntIds

from z3c.form import button, field, group, subform
from z3c.form.interfaces import HIDDEN_MODE, ISubForm, IHandlerForm, \
    IActionHandler
from z3c.formui import form
from z3c.pagelet.browser import BrowserPagelet

from zw.vogon.participant.browser.participants import AddParticipantsForm
from zw.vogon.participant.interfaces import IParticipants

from zw.vogon.survey.attribute import surveyAttributesFactory
from zw.vogon.survey.interfaces import ISurvey, IStoreSurveyAttributes, \
    ISurveyAttributes, IBaseSurveyAttributesSchema
from zw.vogon.template.events import CloneAddedEvent
from zw.vogon.template.interfaces import ISurveyTemplate, ITemplateFactory, \
    ITemplateFolder, ISurveyAttributeTemplates, IStoreSurveyAttributeTemplates


class AttributeGroup(group.Group):

    @Lazy
    def attribute_schema(self):
        if IStoreSurveyAttributeTemplates.providedBy(
            self.parentForm.survey_template):
            return ISurveyAttributeTemplates(
                self.parentForm.survey_template).getSurveyAttributesSchema()
        return IBaseSurveyAttributesSchema

    @Lazy
    def fields(self):
        return field.Fields(self.attribute_schema)
        
    def getContent(self, content=None):
        if content is None:
            content = self.context
        if IStoreSurveyAttributes.providedBy(content):
            content = surveyAttributesFactory(content, self.attribute_schema)
            return content
        return super(AttributeGroup, self).getContent()

    def updateWidgets(self):
        super(AttributeGroup, self).updateWidgets()
        for name in self.attribute_schema:
            if self.attribute_schema[name].readonly:
                self.widgets[name].mode = HIDDEN_MODE


class TemplateSurveyForm(group.GroupForm, form.AddForm):

    content = None
    survey_template = None
    groups = (AttributeGroup,)

    def extractTemplateFromRequest(self):
        intids = getUtility(IIntIds)
        try:
            template_id = int(self.request.get('tid'))
            template = intids.queryObject(template_id)
            assert template is not None

        except:
            pass

        else:
            self.survey_template = template
            self.factory = ITemplateFactory(template)


    def update(self):
        self.extractTemplateFromRequest()

        # If we don't have a valid template redirect to context
        if not ISurveyTemplate.providedBy(self.survey_template):
            self.request.response.redirect(self.request.getURL(1))
            return u''

        self.updateWidgets()

        groups = []
        for groupClass in self.groups:
            group = groupClass(self.context, self.request, self)
            group.update()
            groups.append(group)
        self.groups = tuple(groups)

        self.updateActions()
        self.actions.execute()
        
        # Collect subforms
        subforms = []
        #for question in findObjectsProviding(self.survey_template,
        #                                     IChoiceQuestion):
        #    pass

        # FIXME: Need a general approach
        folder = queryAdapter(self.content, IParticipants, default=self.context)
        add_participants = AddParticipantsSubForm(folder, self.request, self)
        add_participants.prefix = self.prefix + 'add_participants_form.'
        add_participants.buttons = button.Buttons()
        add_participants.update()
        
        self.add_participants = add_participants

    @Lazy
    def fields(self):
        return field.Fields(ISurvey).omit('__name__', '__parent__')

    @property
    def label(self):
        return _( u"Create survey from '${template}'",
                  mapping = { 'template': self.survey_template.title } )

    @button.buttonAndHandler( _( u'Add', ), name='add' )
    def handleAltAdd(self, action):
        self.handleAdd(self, action)

    def create(self, data):
        content = self.factory()
        changed = False
        form.applyChanges(self, content, data)
        for group in self.groups:
            groupContent = group.getContent(content)
            groupChanged = form.applyChanges(group, groupContent, data)
            changed = changed or groupChanged
        
        return content

    def add(self, content):
        name = getattr(content, '__name__', None)
        chooser = INameChooser(self.context)

        # check precondition
        checkObject(self.context, name, content)

        name = chooser.chooseName(name or '', content)
        chooser.checkName(name, content)

        self.context[name] = content
        self.content = content

        #notify(CloneAddedEvent(content, self.survey_template))

        return self.context[name]

    def nextURL(self):
        try:
            failed = self.add_participants.failed
        except AttributeError:
            pass
        else:
            if failed is not None:
                return absoluteURL( self.content, self.request ) + \
                    '/participants.html?failed=%s' % failed 
        return self.request.getURL(1)


    def render(self):
        if self.survey_template is None:
            self.request.response.redirect( self.nextURL() )
            return u''
        return super(TemplateSurveyForm, self).render()


class AddParticipantsSubForm(subform.EditSubForm, AddParticipantsForm):
    implements(ISubForm, IHandlerForm)

    ignoreContext = True
    failed = None

    @button.handler(TemplateSurveyForm.buttons['add'])
    def handlerApply(self, actions):
        data, errors = self.widgets.extract()
        if errors:
            self.status = self.formErrorsMessage
            return
        if self.parentForm.status == self.parentForm.formErrorsMessage:
            return
        self.createAndAdd(data)

        failed = self.widgets['email_addresses'].value
        if failed:
            self.failed = base64.urlsafe_b64encode( failed.encode('utf-8') )
            

        # We couldn't catch any errors...
        #self._finishedAdd = True

    



class TemplatesPortlet(object):

    templates = ()

    def update(self):
        # get
        templates_folder = queryUtility(ITemplateFolder, name=u'Templates')
        intids = getUtility(IIntIds)
        if templates_folder is None:
            return

        templates = []
        for item in templates_folder.templates():
            iid = str( intids.queryId(item) )
            selected = ( iid in self.request.get('tid', '') ) and u'selected' \
                or u''
            templates.append( {
                    'title': item.title,
                    'intid': iid,
                    'selected': selected,
                    } )
        self.templates = tuple(templates)

