#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope.component import adapter, createObject, getAdapter
from zope.component.interfaces import ObjectEvent
from zope.event import notify
from zope.interface import alsoProvides, implements, Interface
from zope.lifecycleevent import ObjectCreatedEvent
from zope.schema import getFieldNames
from zope.app.container.contained import ObjectAddedEvent
from zope.app.container.interfaces import IObjectAddedEvent, IContainer
from zope.app.intid.interfaces import IIntIdAddedEvent

from zw.vogon.portal.interfaces import INewPortalEvent
from zw.vogon.survey.interfaces import ISurvey, ISheetContainer, \
    IStoreMailTexts, IInvitationText, IReminderText
from zw.vogon.template.interfaces import ITemplateFolder, IClonable, \
    IClone, ICloneAddedEvent, IPostCloningEvent, ITemplateFactory


##############################################################################
# setup new folder on evaluation portal creation

@adapter(INewPortalEvent)
def setUpTemplateFolder(event):
    portal = event.object
    sm = portal.getSiteManager()

    templates = createObject( u'zw.vogon.template.TemplateFolder' )
    notify( ObjectCreatedEvent(templates) )
    portal['templates'] = templates
    notify( ObjectAddedEvent(templates) )
    sm.registerUtility( templates, ITemplateFolder, name=u"Templates" )

##############################################################################
# Cloning events


@adapter(IIntIdAddedEvent)
def startCloningSubscriber(event):
    clone = event.object

    if IClonable.providedBy(clone):
        template = IClone(clone).template
        notify(CloneAddedEvent(clone, event, template))

        notify(PostCloningEvent(clone, event, template))

class CloneAddedEvent(ObjectEvent):
    implements(ICloneAddedEvent)

    def __init__(self, object, event, template):
        super(CloneAddedEvent, self).__init__(object)
        self.original_event = event
        self.template = template

class PostCloningEvent(ObjectEvent):
    implements(IPostCloningEvent)

    def __init__(self, object, event, template):
        super(PostCloningEvent, self).__init__(object)
        self.original_event = event
        self.template = template
        

##############################################################################

def copyFields( iface, src, dst, adapt=True, omit=None, select=None ):
    """Copy the fields in the schema iface from src to dst.
    If adapt then also adapt iface to src and dst (default).
    omit and select allows to omit or select a list of fields.
    """
    if adapt:
        src = getAdapter(src, iface)
        dst = getAdapter(dst, iface)

    assert (omit is None) or (select is None), \
        "You must not specify both 'omit' and 'select'."
    names = set( getFieldNames(iface) )
    if omit is not None:
        names.difference_update(omit)
    elif select is not None:
        names.intersection_update(select)

    for name in list(names):
        setattr(dst, name, getattr( src, name, iface[name].default ) )
                
                


@adapter(ISurvey, ICloneAddedEvent)
def updateSurveyAttributes(survey, event):
    # print survey
    print "Survey: ", survey


@adapter(IStoreMailTexts, ICloneAddedEvent)
def updateMailTexts(survey, event):
    for iface in ( IInvitationText, IReminderText ):
        copyFields( iface, event.template, survey )


@adapter(IContainer, ICloneAddedEvent)
def cloneContainer(container, event):
    for name, item in event.template.items():
        factory = ITemplateFactory(item)
        newitem = factory()
        notify(ObjectCreatedEvent(newitem))
        container[name] = newitem


