#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
"""
$Id$
"""
__docformat__ = 'reStructuredText'

from zw.vogon.i18n import MessageFactory as _

from zope import schema
from zope.annotation.interfaces import IAnnotatable
from zope.component.interfaces import IObjectEvent, IFactory
from zope.interface import Attribute, Interface
from zope.app.container.constraints import containers, contains
from zope.app.container.interfaces import IContained, IOrderedContainer

from zw.schema.reference import Reference

##############################################################################
# Abstract template 

class ITemplate(Interface):
    """Abstract interface to declare an object a template.
    """
    
    factory = schema.DottedName(
        title = _( u"Factory to create the templated object" ),
        required = True
        )
    


##############################################################################
#

class ISurveyTemplate(IContained):
    """Stores a survey template.
    """
    containers('zw.vogon.template.interfaces.ITemplateFolder')

    title = schema.TextLine(
        title = _( u'label-template.ITemplate.title',
                   u"Title" ),
        description = _( u'help-template.ITemplate.title',
                         u"Name of this template" ),
        required = True )

    description = schema.Text(
        title = _( u'label-template.ITemplate.description',
                   u"Description" ),
        description = _( u'help-template.ITemplate.description',
                         u"This optional field allows you to give some "
                         u"remarks on this template." ),
        required = False )
        

##############################################################################
# Template folders

class ITemplateContainer(IOrderedContainer):
    """Store templates.
    """
    contains( 'zw.vogon.template.interfaces.ITemplateFolder', 
              'zw.vogon.template.interfaces.ISurveyTemplate' )

class ITemplateFolder(IContained):
    """Store survey templates.
    """
    containers( 'zw.vogon.portal.interfaces.IPortalContainer',
                'zw.vogon.template.interfaces.ITemplateContainer' )

    title = schema.TextLine(
        title = _( u'label-template.ITemplateFolder.title',
                   u"Title" ),
        description = _( u'help-template.ITemplateFolder.title',
                         u"Name of the folder" ),
        default= _( u"Templates" ),
        required = True )

    def templates():
        """Returns all templates.
        """



##############################################################################
# Attributes plugins - no implementation yet

class IStoreSurveyAttributeTemplates(IAnnotatable):
    """Declares an annotation slot for attribute templates.
    """
    
class ISurveyAttributeTemplates(Interface):
    """The folder containing the survey attribute templates.
    """

    def getSurveyAttributesSchema():
        """Creates an survey attributes schema from the containing templates.
        """


class ISurveyAttributeTemplateContainer(IOrderedContainer):
    """Stores survey attribute schemas on templates.
    """
    contains('zw.vogon.template.interfaces.ISurveyAttributeTemplate')

class ISurveyAttributeTemplate(IContained):
    """A survey attribute.
    """
    containers('zw.vogon.template.interfaces.ISurveyAttributeTemplateContainer')

    title = schema.TextLine(
        title = _( u"Title" ),
        required = True )

    description = schema.TextLine(
        title = _( u"Description" ),
        required = False )
    
    required = schema.Bool(
        title = _( u"Required" ),
        required = True,
        default = True )

    public = schema.Bool(
        title = _( u"Public" ),
        required = True,
        default = True )

    readonly = schema.Bool(
        title = _( u"readonly" ),
        required = True,
        default = False )

    default = schema.TextLine(
        title = _( u"Default value" ),
        required = False )

    

##############################################################################
# Cloning

class ICloneAddedEvent(Interface):
    """Notified after the clone has been added.

    Here attribute adapters and contained objects can adapt to.
    """
    
    object = Attribute("The object being cloned.")

    original_event = Attribute("The IntIdAddedEvent related to this event")

    template = Reference(
        title = _( u"The template the object is cloned from." ) )


class IPostCloningEvent(Interface):
    """Notified after cloning has finished.

    Here attribute adapters and contained objects can adapt to.
    """
    
    object = Attribute("The object being cloned.")

    original_event = Attribute("The IntIdAddedEvent related to this event")

    template = Reference(
        title = _( u"The template the object is cloned from." ) )


class IClonable(IAnnotatable):
    """Implemented by clonable objects to allow storing the template
    reference in the object's annotations.
    """

class IClone(Interface):
    """Declares an object to be cloned from a template.
    """

    template = Reference(
        title = _( u"The template the object is cloned from." ) )


class ITemplateFactory(IFactory):
    """A template factory is an adapter to a template which is able
    to create the new type filled with the template values.
    """
    
    template = Attribute( """The templating object.""" )

    def templateAttributes(self):
        """
        """

    def templateSubObjects(self):
        """Try to adapt the ITemplateFactory for all containted items,
        create the templated ones and add them.
        """
