#-*- coding: utf-8 -*-
#############################################################################
#                                                                           #
#   Copyright (c) 2007-2008 Gregor Giesen <vogon@zaehlwerk.net>             #
#                                                                           #
# This program is free software; you can redistribute it and/or modify      #
# it under the terms of the GNU General Public License as published by      #
# the Free Software Foundation; either version 3 of the License, or         #
# (at your option) any later version.                                       #
#                                                                           #
# This program is distributed in the hope that it will be useful,           #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             #
# GNU General Public License for more details.                              #
#                                                                           #
# You should have received a copy of the GNU General Public License         #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.     #
#                                                                           #
#############################################################################
""" Mapping (field) property classes for annotation adapters.
$Id$
"""
__docformat__ = 'reStructuredText'

from zope.component import adapts, adapter
from zope.interface import implementsOnly, implementer
from zope.schema.fieldproperty import FieldProperty

from z3c.form import button, widget
from z3c.form.browser.submit import SubmitWidget
from z3c.form.interfaces import IFormLayer, IFieldWidget

from zw.vogon.utils.browser.interfaces import ICSSButton

class CSSButton(button.Button):
    implementsOnly(ICSSButton)

    klass = FieldProperty(ICSSButton['klass'])

    def __init__(self, *args, **kwargs):
        self.klass = kwargs.pop('klass', u'')
        super(CSSButton, self).__init__(*args, **kwargs)


class CSSButtonAction(button.ButtonAction):
    adapts(IFormLayer, ICSSButton)

    def __init__(self, request, field):
        super(CSSButtonAction, self).__init__(request, field)
        if field.klass:
            self.klass += ' '+field.klass

    
@adapter(ICSSButton, IFormLayer)
@implementer(IFieldWidget)
def CSSButtonFieldWidget(field, request):
    cssbutton = widget.FieldWidget(field, SubmitWidget(request))
    cssbutton.value = field.title
    cssbutton.klass = field.klass
    print cssbutton.klass
    return cssbutton
